﻿/*
Ригин Антон Михайлович, ПИ ФКН НИУ ВШЭ, группа БПИ154.
Курсовая работа на 1-м курсе.
Тема курсовой работы: "Программа нейросетевого решения дифференциальных уравнений в частных производных".
Файл Form1.cs, содержит члены класса формы Form1.
*/

using System;
using System.Threading;
using System.Windows.Forms;
using AForge.Neuro; // Для представления структуры нейросетей используется AForge.NET Framework.
using DiffEqNeuroLibrary; // Для удобства многие классы и методы, связанные с нейросетями, их производными и функциями активации, представлены в виде библиотеки классов DiffEqNeuroLibrary.

namespace CourseWork1
{
    public partial class Form1 : Form
    {
        ActivationNetwork net;

        Thread thread;

        double coeff = 0, coeffx = 0, coeffy = 0, coeffxx = 0, coeffxy = 0, coeffyx = 0, coeffyy = 0, freeConst = 0;
        double xmin = 0, ymin = 0, xmax = 0, ymax = 0, borderConst = 0;
        double gradStep = 0.000005;
        double pbStep = 0, pbValue = 0;

        double[][] neuronOutputs, differentsX, differentsY, differentsXX, differentsXY, differentsYX, differentsYY;

        double[][] activFuncDerivatives, activFuncSecondDerivatives, activFuncThirdDerivatives;

        bool[][] isActivFuncDerivativeReady, isActivFuncSecondDerivativeReady, isActivFuncThirdDerivativeReady;

        int freeFunctionIndex = 0, borderFunctionIndex = 0;
        int xPointsCount = 10, yPointsCount = 10;
        int iterations = 1500;

        public Form1()
        {
            InitializeComponent();
        }

        /// <summary>
        /// Вычисляет значение левой части уравнения в заданной точке.
        /// </summary>
        /// <param name="x">Координата X заданной точки.</param>
        /// <param name="y">Координата Y заданной точки.</param>
        /// <returns>Возвращает вычисленное значение левой части уравнения в заданной точке.</returns>
        double ComputeDiffEqValue(double x, double y)
        {
            double funcValue = 0, diffXValue = 0, diffYValue = 0, diffXXValue = 0, diffXYValue = 0, diffYXValue = 0, diffYYValue = 0;
            if(coeff != 0)
            {
                funcValue = ComputeFuncValue(x, y);
            }
            if(coeffx != 0)
            {
                diffXValue = NetDifferentiating.NetDifferentX(net, x, y);
            }
            if (coeffy != 0)
            {
                diffYValue = NetDifferentiating.NetDifferentY(net, x, y);
            }
            if (coeffxx != 0)
            {
                diffXXValue = NetDifferentiating.NetDifferentXX(net, x, y);
            }
            if (coeffxy != 0)
            {
                diffXYValue = NetDifferentiating.NetDifferentXY(net, x, y);
            }
            if (coeffyx != 0)
            {
                diffYXValue = NetDifferentiating.NetDifferentYX(net, x, y);
            }
            if (coeffyy != 0)
            {
                diffYYValue = NetDifferentiating.NetDifferentYY(net, x, y);
            }
            double freeFuncValue = ComputeFreeFuncValue(freeFunctionIndex, x, y);
            return coeff * funcValue + coeffx * diffXValue + coeffy * diffYValue + coeffxx * diffXXValue + coeffxy * diffXYValue + coeffyx * diffYXValue + coeffyy * diffYYValue + freeFuncValue + freeConst;
        }

        /// <summary>
        /// Вычисляет значение левой части уравнения в заданной точке с использованием ранее посчитанных выходов и производных нейронов нейросети.
        /// </summary>
        /// <param name="x">Координата X заданной точки.</param>
        /// <param name="y">Координата Y заданной точки.</param>
        /// <returns>Возвращает вычисленное значение левой части уравнения в заданной точке.</returns>
        double ComputeDiffEqValueFromOutputsAndDifferents(double x, double y)
        {
            double funcValue = 0, diffXValue = 0, diffYValue = 0, diffXXValue = 0, diffXYValue = 0, diffYXValue = 0, diffYYValue = 0;
            if (coeff != 0)
            {
                funcValue = neuronOutputs[net.Layers.Length - 1][0];
            }
            if (coeffx != 0)
            {
                diffXValue = differentsX[net.Layers.Length - 1][0];
            }
            if (coeffy != 0)
            {
                diffYValue = differentsY[net.Layers.Length - 1][0];
            }
            if (coeffxx != 0)
            {
                diffXXValue = differentsXX[net.Layers.Length - 1][0];
            }
            if (coeffxy != 0)
            {
                diffXYValue = differentsXY[net.Layers.Length - 1][0];
            }
            if (coeffyx != 0)
            {
                diffYXValue = differentsYX[net.Layers.Length - 1][0];
            }
            if (coeffyy != 0)
            {
                diffYYValue = differentsYY[net.Layers.Length - 1][0];
            }
            return coeff * funcValue + coeffx * diffXValue + coeffy * diffYValue + coeffxx * diffXXValue + coeffxy * diffXYValue + coeffyx * diffYXValue + coeffyy * diffYYValue + ComputeFreeFuncValue(freeFunctionIndex, x, y) + freeConst;
        }

        /// <summary>
        /// Вычисляет значение функции из comboBoxFreeFunction (свободная функция уравнения) или comboBoxBorderFunction (функция граничных условий) в заданной точке.
        /// </summary>
        /// <param name="index">Индекс выбранного элемента из соответствующего ComboBox.</param>
        /// <param name="x">Координата X заданной точки.</param>
        /// <param name="y">Координата Y заданной точки.</param>
        /// <returns>Возвращает вычисленное значение функции из comboBoxFreeFunction (свободная функция уравнения) или comboBoxBorder (функция граничных условий) в заданной точке.</returns>
        double ComputeFreeFuncValue(int index, double x, double y)
        {
            switch(index)
            {
                case 0: return 0;
                case 1: return x;
                case 2: return y;
                case 3: return x + y;
                case 4: return x - y;
                case 5: return y - x;
                case 6: return x * y;
                case 7: return Math.Abs(x);
                case 8: return Math.Abs(y);
                case 9: return Math.Abs(x + y);
                case 10: return Math.Abs(x - y);
                case 11: return Math.Abs(y - x);
                case 12: return Math.Abs(x * y);
                case 13: return Math.Cos(x);
                case 14: return Math.Cos(y);
                case 15: return Math.Cos(x + y);
                case 16: return Math.Cos(x - y);
                case 17: return Math.Cos(y - x);
                case 18: return Math.Cos(x * y);
                case 19: return Math.Exp(x);
                case 20: return Math.Exp(y);
                case 21: return Math.Exp(x + y);
                case 22: return Math.Exp(x - y);
                case 23: return Math.Exp(y - x);
                case 24: return Math.Exp(x * y);
                case 25: return Math.Pow(x, y);
                case 26: return Math.Pow(y, x);
                case 27: return Math.Sign(x);
                case 28: return Math.Sign(y);
                case 29: return Math.Sign(x + y);
                case 30: return Math.Sign(x - y);
                case 31: return Math.Sign(y - x);
                case 32: return Math.Sign(x * y);
                case 33: return Math.Sin(x);
                case 34: return Math.Sin(y);
                case 35: return Math.Sin(x + y);
                case 36: return Math.Sin(x - y);
                case 37: return Math.Sin(y - x);
                case 38: return Math.Sin(x * y);
                case 39: return x * x;
                case 40: return y * y;
                case 41: return (x + y) * (x + y);
                case 42: return (x - y) * (x - y);
                case 43: return (y - x) * (y - x);
                case 44: return (x * y) * (x * y);
                default: return 0;
            }
        }

        /// <summary>
        /// Вычисляет значение искомой функции (заданной нейросетью) в заданной точке.
        /// </summary>
        /// <param name="x">Координата X заданной точки.</param>
        /// <param name="y">Координата Y заданной точки.</param>
        /// <returns>Возвращает вычисленное значение искомой функции (заданной нейросетью) в заданной точке.</returns>
        double ComputeFuncValue(double x, double y)
        {
            double[] input = { x, y };
            double[] output = net.Compute(input);
            return output[0];
        }

        /// <summary>
        /// Вычисляет максимальную ошибку на левой части уравнения.
        /// </summary>
        /// <param name="maxBorderMistake">Максимальная ошибка на граничных условиях.</param>
        /// <returns>Возвращает вычисленную максимальную ошибку на левой части уравнения.</returns>
        double ComputeNetMaxMistake(out double maxBorderMistake)
        {
            double maxMistake = 0;
            maxBorderMistake = 0;
            double xstep = (xmax - xmin) / (2 * xPointsCount), ystep = (ymax - ymin) / (2 * yPointsCount);
            for (int xi = 0; xi <= 2 * xPointsCount; xi++)
            {
                for (int yi = 0; yi <= 2 * yPointsCount; yi++)
                {
                    double x = xmin + xi * xstep, y = ymin + yi * ystep;
                    double mistake = Math.Abs(ComputeDiffEqValue(x, y));
                    if(mistake > maxMistake)
                    {
                        maxMistake = mistake;
                    }
                    if (xi == 0 || yi == 0 || xi == xPointsCount || yi == yPointsCount)
                    {
                        double borderMistake = Math.Abs(ComputeFuncValue(x, y) - (ComputeFreeFuncValue(borderFunctionIndex, x, y) + borderConst));
                        if(borderMistake > maxBorderMistake)
                        {
                            maxBorderMistake = borderMistake;
                        }
                    }
                }
            }
            return maxMistake;
        }

        /// <summary>
        /// Вычисляет выходы и производные нейронов нейросети в заданной точке.
        /// </summary>
        /// <param name="x">Координата X заданной точки.</param>
        /// <param name="y">Координата Y заданной точки.</param>
        void ComputeOutputsAndDifferents(double x, double y)
        {
            activFuncDerivatives = new double[net.Layers.Length][];
            isActivFuncDerivativeReady = new bool[net.Layers.Length][];
            for(int i = 0; i < net.Layers.Length; i++)
            {
                activFuncDerivatives[i] = new double[net.Layers[i].Neurons.Length];
                isActivFuncDerivativeReady[i] = new bool[net.Layers[i].Neurons.Length];
            }
            neuronOutputs = NetDifferentiating.ComputeNeuronOutputs(net, x, y);
            if(coeffx != 0 || coeffy != 0 || coeffxx != 0 || coeffxy != 0 || coeffyx != 0 || coeffyy != 0)
            {
                activFuncSecondDerivatives = new double[net.Layers.Length][];
                isActivFuncSecondDerivativeReady = new bool[net.Layers.Length][];
                for (int i = 0; i < net.Layers.Length; i++)
                {
                    activFuncSecondDerivatives[i] = new double[net.Layers[i].Neurons.Length];
                    isActivFuncSecondDerivativeReady[i] = new bool[net.Layers[i].Neurons.Length];
                }
            }
            if(coeffxx != 0 || coeffxy != 0 || coeffyx != 0 || coeffyy != 0)
            {
                activFuncThirdDerivatives = new double[net.Layers.Length][];
                isActivFuncThirdDerivativeReady = new bool[net.Layers.Length][];
                for (int i = 0; i < net.Layers.Length; i++)
                {
                    activFuncThirdDerivatives[i] = new double[net.Layers[i].Neurons.Length];
                    isActivFuncThirdDerivativeReady[i] = new bool[net.Layers[i].Neurons.Length];
                }
            }
            if(coeffx != 0 || coeffxx != 0 || coeffxy != 0)
            {
                ArctgActivationFunction arctgFunc = new ArctgActivationFunction();
                differentsX = NetDifferentiating.DifferentsX(net, arctgFunc, neuronOutputs, ref activFuncDerivatives, ref isActivFuncDerivativeReady, x, y);
            }
            if(coeffy != 0 || coeffyx != 0 || coeffyy != 0)
            {
                ArctgActivationFunction arctgFunc = new ArctgActivationFunction();
                differentsY = NetDifferentiating.DifferentsY(net, arctgFunc, neuronOutputs, ref activFuncDerivatives, ref isActivFuncDerivativeReady, x, y);
            }
            if(coeffxx != 0)
            {
                ArctgActivationFunction arctgFunc = new ArctgActivationFunction();
                differentsXX = NetDifferentiating.DifferentsXX(net, arctgFunc, neuronOutputs, differentsX, ref activFuncDerivatives, ref isActivFuncDerivativeReady, ref activFuncSecondDerivatives, ref isActivFuncSecondDerivativeReady, x, y);
            }
            if (coeffxy != 0)
            {
                ArctgActivationFunction arctgFunc = new ArctgActivationFunction();
                differentsXY = NetDifferentiating.DifferentsXY(net, arctgFunc, neuronOutputs, differentsX, ref activFuncDerivatives, ref isActivFuncDerivativeReady, ref activFuncSecondDerivatives, ref isActivFuncSecondDerivativeReady, x, y);
            }
            if (coeffyx != 0)
            {
                ArctgActivationFunction arctgFunc = new ArctgActivationFunction();
                differentsYX = NetDifferentiating.DifferentsYX(net, arctgFunc, neuronOutputs, differentsY, ref activFuncDerivatives, ref isActivFuncDerivativeReady, ref activFuncSecondDerivatives, ref isActivFuncSecondDerivativeReady, x, y);
            }
            if (coeffyy != 0)
            {
                ArctgActivationFunction arctgFunc = new ArctgActivationFunction();
                differentsYY = NetDifferentiating.DifferentsYY(net, arctgFunc, neuronOutputs, differentsY, ref activFuncDerivatives, ref isActivFuncDerivativeReady, ref activFuncSecondDerivatives, ref isActivFuncSecondDerivativeReady, x, y);
            }
        }

        /// <summary>
        /// Вычисляет производную левой части уравнения по заданному весовому коэффициенту в заданной точке.
        /// </summary>
        /// <param name="layerNumber">Номер слоя заданного весового коэффициента.</param>
        /// <param name="neuronInLayerNumber">Номер нейрона заданного весового коэффициента в слое.</param>
        /// <param name="weightInNeuronNumber">Номер заданного весового коэффициента в нейроне.</param>
        /// <param name="x">Координата X заданной точки.</param>
        /// <param name="y">Координата Y заданной точки.</param>
        /// <returns>Возвращает вычисленную производную левой части уравнения по заданному весовому коэффициенту в заданной точке.</returns>
        double ComputeWeightInDiffEqDifferent(int layerNumber, int neuronInLayerNumber, int weightInNeuronNumber, double x, double y)
        {
            double funcWeightDifferent = 0, diffXWeightDifferent = 0, diffYWeightDifferent = 0, diffXXWeightDifferent = 0, diffXYWeightDifferent = 0, diffYXWeightDifferent = 0, diffYYWeightDifferent = 0;
            if (coeff != 0)
            {
                funcWeightDifferent = NetDifferentiating.NetDifferentWeight(net, neuronOutputs, ref activFuncDerivatives, ref isActivFuncDerivativeReady, layerNumber, neuronInLayerNumber, weightInNeuronNumber, x, y);
            }
            if (coeffx != 0)
            {
                diffXWeightDifferent = NetDifferentiating.NetDifferentXWeight(net, neuronOutputs, differentsX, ref activFuncDerivatives, ref isActivFuncDerivativeReady, ref activFuncSecondDerivatives, ref isActivFuncSecondDerivativeReady, layerNumber, neuronInLayerNumber, weightInNeuronNumber, x, y);
            }
            if (coeffy != 0)
            {
                diffYWeightDifferent = NetDifferentiating.NetDifferentYWeight(net, neuronOutputs, differentsY, ref activFuncDerivatives, ref isActivFuncDerivativeReady, ref activFuncSecondDerivatives, ref isActivFuncSecondDerivativeReady, layerNumber, neuronInLayerNumber, weightInNeuronNumber, x, y);
            }
            if (coeffxx != 0)
            {
                diffXXWeightDifferent = NetDifferentiating.NetDifferentXXWeight(net, neuronOutputs, differentsX, differentsXX, ref activFuncDerivatives, ref isActivFuncDerivativeReady, ref activFuncSecondDerivatives, ref isActivFuncSecondDerivativeReady, ref activFuncThirdDerivatives, ref isActivFuncThirdDerivativeReady, layerNumber, neuronInLayerNumber, weightInNeuronNumber, x, y);
            }
            if (coeffxy != 0)
            {
                diffXYWeightDifferent = NetDifferentiating.NetDifferentXYWeight(net, neuronOutputs, differentsX, differentsXY, ref activFuncDerivatives, ref isActivFuncDerivativeReady, ref activFuncSecondDerivatives, ref isActivFuncSecondDerivativeReady, ref activFuncThirdDerivatives, ref isActivFuncThirdDerivativeReady, layerNumber, neuronInLayerNumber, weightInNeuronNumber, x, y);
            }
            if (coeffyx != 0)
            {
                diffYXWeightDifferent = NetDifferentiating.NetDifferentYXWeight(net, neuronOutputs, differentsY, differentsYX, ref activFuncDerivatives, ref isActivFuncDerivativeReady, ref activFuncSecondDerivatives, ref isActivFuncSecondDerivativeReady, ref activFuncThirdDerivatives, ref isActivFuncThirdDerivativeReady, layerNumber, neuronInLayerNumber, weightInNeuronNumber, x, y);
            }
            if (coeffyy != 0)
            {
                diffYYWeightDifferent = NetDifferentiating.NetDifferentYYWeight(net, neuronOutputs, differentsY, differentsYY, ref activFuncDerivatives, ref isActivFuncDerivativeReady, ref activFuncSecondDerivatives, ref isActivFuncSecondDerivativeReady, ref activFuncThirdDerivatives, ref isActivFuncThirdDerivativeReady, layerNumber, neuronInLayerNumber, weightInNeuronNumber, x, y);
            }
            return coeff * funcWeightDifferent + coeffx * diffXWeightDifferent + coeffy * diffYWeightDifferent + coeffxx * diffXXWeightDifferent + coeffxy * diffXYWeightDifferent + coeffyx * diffYXWeightDifferent + coeffyy * diffYYWeightDifferent;
        }

        /// <summary>
        /// Выполняет процесс обучения нейросети.
        /// </summary>
        void RunNetworkTraining()
        {
            for (int i = 0; i < iterations; i++)
            {
                RunNetworkTrainingIteration();
                Invoke(new ThreadStart(delegate
                {
                    labelProgress.Text = "Прогресс выполнения - Выполнено итераций " + (i + 1) + " из " + iterations;
                    pbValue += pbStep;
                    if (pbValue < progressBar1.Maximum)
                    {
                        progressBar1.Value = (int)pbValue;
                    }
                }));
            }
            double maxBorderMistake;
            double maxMistake = ComputeNetMaxMistake(out maxBorderMistake);
            Invoke(new ThreadStart(delegate
            {
                progressBar1.Value = progressBar1.Maximum;
                buttonAbort.Enabled = false;
                FormGraph formGraph = new FormGraph(net, xmin, ymin, xmax, ymax, maxMistake, maxBorderMistake);
                formGraph.ShowDialog();
                buttonStart.Enabled = true;
            }));
        }

        /// <summary>
        /// Выполняет одну итерацию обучения нейросети.
        /// </summary>
        void RunNetworkTrainingIteration()
        {
            /*double[][][] differentEq = new double[net.Layers.Length][][], differentBorder = new double[net.Layers.Length][][];
            for (int i = 0; i < net.Layers.Length; i++)
            {
                differentEq[i] = new double[net.Layers[i].Neurons.Length][];
                differentBorder[i] = new double[net.Layers[i].Neurons.Length][];
                for (int j = 0; j < net.Layers[i].Neurons.Length; j++)
                {
                    differentEq[i][j] = new double[net.Layers[i].Neurons[j].Weights.Length + 1];
                    differentBorder[i][j] = new double[net.Layers[i].Neurons[j].Weights.Length + 1];
                    for (int k = 0; k < net.Layers[i].Neurons[j].Weights.Length + 1; k++)
                    {
                        differentEq[i][j][k] = 0;
                        differentBorder[i][j][k] = 0;
                        double xstep = (xmax - xmin) / xPointsCount, ystep = (ymax - ymin) / yPointsCount;
                        for (int xi = 0; xi <= xPointsCount; xi++)
                        {
                            for (int yi = 0; yi <= yPointsCount; yi++)
                            {
                                double x = xmin + xi * xstep, y = ymin + yi * ystep;
                                differentEq[i][j][k] += ComputeDiffEqValue(x, y) * ComputeWeightInDiffEqDifferent(i, j, k, x, y);
                                double diffEqValue = ComputeDiffEqValue(x, y);
                                if (xi == 0 || yi == 0 || xi == xPointsCount || yi == yPointsCount)
                                {
                                    differentBorder[i][j][k] += ComputeFuncValue(x, y) * NetDifferentiating.NetDifferentWeight(net, i, j, k, x, y);
                                }
                            }
                        }
                    }
                }
            }
            for (int i = 0; i < net.Layers.Length; i++)
            {
                for (int j = 0; j < net.Layers[i].Neurons.Length; j++)
                {
                    for (int k = 0; k <= net.Layers[i].Neurons[j].Weights.Length; k++)
                    {
                        if (k < net.Layers[i].Neurons[j].Weights.Length)
                        {
                            net.Layers[i].Neurons[j].Weights[k] -= gradStep * (differentEq[i][j][k] + differentBorder[i][j][k]);
                        }
                        else
                        {
                            ActivationNeuron neuron = (ActivationNeuron)net.Layers[i].Neurons[j];
                            neuron.Threshold -= gradStep * (differentEq[i][j][k] + differentBorder[i][j][k]);
                            net.Layers[i].Neurons[j] = neuron;
                        }
                    }
                }
            }*/
            double[][][] differentEq = new double[net.Layers.Length][][], differentBorder = new double[net.Layers.Length][][];
            for (int i = 0; i < net.Layers.Length; i++)
            {
                differentEq[i] = new double[net.Layers[i].Neurons.Length][];
                differentBorder[i] = new double[net.Layers[i].Neurons.Length][];
                for (int j = 0; j < net.Layers[i].Neurons.Length; j++)
                {
                    differentEq[i][j] = new double[net.Layers[i].Neurons[j].Weights.Length + 1];
                    differentBorder[i][j] = new double[net.Layers[i].Neurons[j].Weights.Length + 1];
                    for (int k = 0; k < net.Layers[i].Neurons[j].Weights.Length + 1; k++)
                    {
                        differentEq[i][j][k] = 0;
                        differentBorder[i][j][k] = 0;
                    }
                }
            }
            double xstep = (xmax - xmin) / xPointsCount, ystep = (ymax - ymin) / yPointsCount;
            for (int xi = 0; xi <= xPointsCount; xi++)
            {
                double x = xmin + xi * xstep;
                for (int yi = 0; yi <= yPointsCount; yi++)
                {
                    double y = ymin + yi * ystep;
                    ComputeOutputsAndDifferents(x, y);
                    for (int i = 0; i < net.Layers.Length; i++)
                    {
                        for (int j = 0; j < net.Layers[i].Neurons.Length; j++)
                        {
                            for (int k = 0; k < net.Layers[i].Neurons[j].Weights.Length + 1; k++)
                            {
                                differentEq[i][j][k] += ComputeDiffEqValueFromOutputsAndDifferents(x, y) * ComputeWeightInDiffEqDifferent(i, j, k, x, y);
                                if (xi == 0 || yi == 0 || xi == xPointsCount || yi == yPointsCount)
                                {
                                    differentBorder[i][j][k] += (neuronOutputs[net.Layers.Length - 1][0] - (ComputeFreeFuncValue(borderFunctionIndex, x, y) + borderConst)) * NetDifferentiating.NetDifferentWeight(net, neuronOutputs, ref activFuncDerivatives, ref isActivFuncDerivativeReady, i, j, k, x, y);
                                }
                            }
                        }
                    }
                }
            }
            for (int i = 0; i < net.Layers.Length; i++)
            {
                for (int j = 0; j < net.Layers[i].Neurons.Length; j++)
                {
                    for (int k = 0; k <= net.Layers[i].Neurons[j].Weights.Length; k++)
                    {
                        if (k < net.Layers[i].Neurons[j].Weights.Length)
                        {
                            net.Layers[i].Neurons[j].Weights[k] -= gradStep * (differentEq[i][j][k] + differentBorder[i][j][k]);
                        }
                        else
                        {
                            ActivationNeuron neuron = (ActivationNeuron)net.Layers[i].Neurons[j];
                            neuron.Threshold -= gradStep * (differentEq[i][j][k] + differentBorder[i][j][k]);
                            net.Layers[i].Neurons[j] = neuron;
                        }
                    }
                }
            }
        }

        private void buttonAbort_Click(object sender, EventArgs e)
        {
            thread.Abort();
            labelProgress.Text = "Прогресс выполнения - процесс прерван";
            buttonAbort.Enabled = false;
            buttonStart.Enabled = true;
            progressBar1.Value = progressBar1.Minimum;
        }

        private void buttonStart_Click(object sender, EventArgs e)
        {
            if (!double.TryParse(textBoxf.Text, out coeff))
            {
                string errorText = "Некорректный формат коэффициента при f";
                FormErrorDialog formErrorDialog = new FormErrorDialog(errorText);
                formErrorDialog.ShowDialog();
                return;
            }
            if (!double.TryParse(textBoxfx.Text, out coeffx))
            {
                string errorText = "Некорректный формат коэффициента при f'x";
                FormErrorDialog formErrorDialog = new FormErrorDialog(errorText);
                formErrorDialog.ShowDialog();
                return;
            }
            if (!double.TryParse(textBoxfy.Text, out coeffy))
            {
                string errorText = "Некорректный формат коэффициента при f'y";
                FormErrorDialog formErrorDialog = new FormErrorDialog(errorText);
                formErrorDialog.ShowDialog();
                return;
            }
            if (!double.TryParse(textBoxfxx.Text, out coeffxx))
            {
                string errorText = "Некорректный формат коэффициента при f''xx";
                FormErrorDialog formErrorDialog = new FormErrorDialog(errorText);
                formErrorDialog.ShowDialog();
                return;
            }
            if (!double.TryParse(textBoxfxy.Text, out coeffxy))
            {
                string errorText = "Некорректный формат коэффициента при f''xy";
                FormErrorDialog formErrorDialog = new FormErrorDialog(errorText);
                formErrorDialog.ShowDialog();
                return;
            }
            if (!double.TryParse(textBoxfyx.Text, out coeffyx))
            {
                string errorText = "Некорректный формат коэффициента при f''yx";
                FormErrorDialog formErrorDialog = new FormErrorDialog(errorText);
                formErrorDialog.ShowDialog();
                return;
            }
            if (!double.TryParse(textBoxfyy.Text, out coeffyy))
            {
                string errorText = "Некорректный формат коэффициента при f''yy";
                FormErrorDialog formErrorDialog = new FormErrorDialog(errorText);
                formErrorDialog.ShowDialog();
                return;
            }
            if(comboBoxFreeFunction.SelectedIndex < 0)
            {
                string errorText = "Выберите свободную функцию";
                FormErrorDialog formErrorDialog = new FormErrorDialog(errorText);
                formErrorDialog.ShowDialog();
                return;
            }
            if (!double.TryParse(textBoxFreeConst.Text, out freeConst))
            {
                string errorText = "Некорректный формат свободной константы";
                FormErrorDialog formErrorDialog = new FormErrorDialog(errorText);
                formErrorDialog.ShowDialog();
                return;
            }
            if (!double.TryParse(textBoxXmin.Text, out xmin))
            {
                string errorText = "Некорректный формат Xmin";
                FormErrorDialog formErrorDialog = new FormErrorDialog(errorText);
                formErrorDialog.ShowDialog();
                return;
            }
            if (!double.TryParse(textBoxYmin.Text, out ymin))
            {
                string errorText = "Некорректный формат Ymin";
                FormErrorDialog formErrorDialog = new FormErrorDialog(errorText);
                formErrorDialog.ShowDialog();
                return;
            }
            if (!double.TryParse(textBoxXmax.Text, out xmax))
            {
                string errorText = "Некорректный формат Xmax";
                FormErrorDialog formErrorDialog = new FormErrorDialog(errorText);
                formErrorDialog.ShowDialog();
                return;
            }
            if (!double.TryParse(textBoxYmax.Text, out ymax))
            {
                string errorText = "Некорректный формат Ymax";
                FormErrorDialog formErrorDialog = new FormErrorDialog(errorText);
                formErrorDialog.ShowDialog();
                return;
            }
            if(comboBoxXPointsCount.SelectedIndex < 0)
            {
                string errorText = "Выберите количество точек по X";
                FormErrorDialog formErrorDialog = new FormErrorDialog(errorText);
                formErrorDialog.ShowDialog();
                return;
            }
            if (comboBoxYPointsCount.SelectedIndex < 0)
            {
                string errorText = "Выберите количество точек по Y";
                FormErrorDialog formErrorDialog = new FormErrorDialog(errorText);
                formErrorDialog.ShowDialog();
                return;
            }
            if (comboBoxBorderFunction.SelectedIndex < 0)
            {
                string errorText = "Выберите функцию на граничных условиях";
                FormErrorDialog formErrorDialog = new FormErrorDialog(errorText);
                formErrorDialog.ShowDialog();
                return;
            }
            if (!double.TryParse(textBoxBorderConst.Text, out borderConst))
            {
                string errorText = "Некорректный формат константы на граничных условиях";
                FormErrorDialog formErrorDialog = new FormErrorDialog(errorText);
                formErrorDialog.ShowDialog();
                return;
            }
            if(comboBoxLayer1.SelectedIndex < 0)
            {
                string errorText = "Выберите количество нейронов в слое 1";
                FormErrorDialog formErrorDialog = new FormErrorDialog(errorText);
                formErrorDialog.ShowDialog();
                return;
            }
            if (comboBoxLayer2.SelectedIndex < 0)
            {
                string errorText = "Выберите количество нейронов в слое 2";
                FormErrorDialog formErrorDialog = new FormErrorDialog(errorText);
                formErrorDialog.ShowDialog();
                return;
            }
            if (comboBoxLayer3.SelectedIndex < 0)
            {
                string errorText = "Выберите количество нейронов в слое 3";
                FormErrorDialog formErrorDialog = new FormErrorDialog(errorText);
                formErrorDialog.ShowDialog();
                return;
            }
            if(comboBoxGrad.SelectedIndex < 0)
            {
                string errorText = "Выберите шаг градиентного спуска";
                FormErrorDialog formErrorDialog = new FormErrorDialog(errorText);
                formErrorDialog.ShowDialog();
                return;
            }
            if (comboBoxIterations.SelectedIndex < 0)
            {
                string errorText = "Выберите количество итераций";
                FormErrorDialog formErrorDialog = new FormErrorDialog(errorText);
                formErrorDialog.ShowDialog();
                return;
            }
            if(xmax <= xmin)
            {
                string errorText = "Xmin должно быть меньше Xmax";
                FormErrorDialog formErrorDialog = new FormErrorDialog(errorText);
                formErrorDialog.ShowDialog();
                return;
            }
            if (ymax <= ymin)
            {
                string errorText = "Ymin должно быть меньше Ymax";
                FormErrorDialog formErrorDialog = new FormErrorDialog(errorText);
                formErrorDialog.ShowDialog();
                return;
            }
            freeFunctionIndex = comboBoxFreeFunction.SelectedIndex;
            xPointsCount = comboBoxXPointsCount.SelectedIndex + 5;
            yPointsCount = comboBoxYPointsCount.SelectedIndex + 5;
            borderFunctionIndex = comboBoxBorderFunction.SelectedIndex;
            int layer1Count = comboBoxLayer1.SelectedIndex + 1;
            int layer2Count = comboBoxLayer2.SelectedIndex + 1;
            int layer3Count = comboBoxLayer3.SelectedIndex + 1;
            gradStep = double.Parse(comboBoxGrad.Text);
            iterations = (comboBoxIterations.SelectedIndex + 1) * 500;
            ArctgActivationFunction arctgFunc = new ArctgActivationFunction();
            net = new ActivationNetwork(arctgFunc, 2, layer1Count, layer2Count, layer3Count, 1);
            LinearActivationFunction linFunc = new LinearActivationFunction();
            ActivationLayer outputLayer = new ActivationLayer(1, layer3Count, linFunc);
            net.Layers[net.Layers.Length - 1] = outputLayer;
            pbStep = ((double)progressBar1.Maximum) / iterations;
            pbValue = progressBar1.Minimum;
            buttonStart.Enabled = false;
            buttonAbort.Enabled = true;
            thread = new Thread(RunNetworkTraining);
            thread.Start();
        }

        private void Form1_FormClosing(object sender, FormClosingEventArgs e)
        {
            if (thread != null)
            {
                thread.Abort();
            }
        }
    }
}
