﻿/*
Ригин Антон Михайлович, ПИ ФКН НИУ ВШЭ, группа БПИ154.
Курсовая работа на 1-м курсе.
Тема курсовой работы: "Программа нейросетевого решения дифференциальных уравнений в частных производных".
Файл FormGraph.cs, содержит члены класса формы FormGraph.
*/

using System;
using System.Drawing;
using System.Windows.Forms;
using AForge.Neuro; // Для представления структуры нейросетей используется AForge.NET Framework.

namespace CourseWork1
{
    public partial class FormGraph : Form
    {
        ActivationNetwork net;

        Color[][] colors;

        Color[] paletteColors;

        bool isPaletteReady = false;

        double[][] funcValues;

        double funcMin = 0, funcMax = 0;
        double xStep = 0, yStep = 0;
        double xmin = 0, ymin = 0, xmax = 0, ymax = 0;
        
        public FormGraph(ActivationNetwork net, double xmin, double ymin, double xmax, double ymax, double maxMistake, double maxBorderMistake)
        {
            InitializeComponent();
            this.net = net;
            this.xmin = xmin;
            this.ymin = ymin;
            this.xmax = xmax;
            this.ymax = ymax;
            labelMaxMistake.Text = "Макс. ошибка: " + maxMistake;
            labelMaxBorderMistake.Text = "Макс. ошибка на гран. условиях: " + maxBorderMistake;
            labelMaxMistake.Visible = true;
            labelMaxBorderMistake.Visible = true;
            InitializeFunction();
        }

        /// <summary>
        /// Вычисляет значение искомой функции (заданной нейросетью) в заданной точке.
        /// </summary>
        /// <param name="x">Координата X заданной точки.</param>
        /// <param name="y">Координата Y заданной точки.</param>
        /// <returns>Возвращает вычисленное значение искомой функции (заданной нейросетью) в заданной точке.</returns>
        double ComputeFuncValue(double x, double y)
        {
            double[] input = { x, y };
            double[] output = net.Compute(input);
            return output[0];
        }

        /// <summary>
        /// Вычисляет значения функции и цветов во всех точках отрисовываемой области и цветов для палитры, а также вызывает отрисовку графика функции и палитры.
        /// </summary>
        void InitializeFunction()
        {
            Cursor.Current = Cursors.WaitCursor;
            funcValues = new double[pictureBox1.Size.Width][];
            colors = new Color[pictureBox1.Size.Width][];
            for (int xi = 0; xi < pictureBox1.Size.Width; xi++)
            {
                funcValues[xi] = new double[pictureBox1.Size.Height];
                colors[xi] = new Color[pictureBox1.Size.Height];
            }
            xStep = (xmax - xmin) / pictureBox1.Size.Width;
            yStep = (ymax - ymin) / pictureBox1.Size.Height;
            funcMin = ComputeFuncValue(xmin, ymin);
            funcMax = funcMin;
            for (int xi = 0; xi < pictureBox1.Size.Width; xi++)
            {
                double x = xmin + xi * xStep;
                for (int yi = 0; yi < pictureBox1.Size.Height; yi++)
                {
                    double y = ymin + yi * yStep;
                    funcValues[xi][yi] = ComputeFuncValue(x, y);
                    if (funcValues[xi][yi] < funcMin)
                    {
                        funcMin = funcValues[xi][yi];
                    }
                    if (funcValues[xi][yi] > funcMax)
                    {
                        funcMax = funcValues[xi][yi];
                    }
                }
            }
            for (int xi = 0; xi < pictureBox1.Size.Width; xi++)
            {
                for (int yi = 0; yi < pictureBox1.Size.Height; yi++)
                {
                    /*int colorNumber = (int)(16777216 * ((funcValues[xi][yi] - funcMin) / (funcMax - funcMin)));
                    string hexColorNumber = Convert.ToString(colorNumber, 16);
                    int n = 6 - hexColorNumber.Length;
                    for (int i = 0; i < n; i++)
                    {
                        hexColorNumber = "0" + hexColorNumber;
                    }
                    int red = Convert.ToInt32(hexColorNumber.Substring(0, 2), 16);
                    int green = Convert.ToInt32(hexColorNumber.Substring(2, 2), 16);
                    int blue = Convert.ToInt32(hexColorNumber.Substring(4, 2), 16);
                    colors[xi][yi] = Color.FromArgb(red, green, blue);*/
                    double colorRelation = 255 * ((funcValues[xi][yi] - funcMin) / (funcMax - funcMin));
                    int red = (int)(colorRelation);
                    int green = (int)(255 - colorRelation);
                    int blue = 255;
                    colors[xi][yi] = Color.FromArgb(red, green, blue);
                }
            }
            if (!isPaletteReady)
            {
                paletteColors = new Color[pictureBoxPalette.Size.Width];
                for (int xi = 0; xi < pictureBoxPalette.Size.Width; xi++)
                {
                    /*int colorNumber = (int)(16777216 * (((double)xi) / pictureBoxPalette.Size.Width));
                    string hexColorNumber = Convert.ToString(colorNumber, 16);
                    int n = 6 - hexColorNumber.Length;
                    for (int i = 0; i < n; i++)
                    {
                        hexColorNumber = "0" + hexColorNumber;
                    }
                    int red = Convert.ToInt32(hexColorNumber.Substring(0, 2), 16);
                    int green = Convert.ToInt32(hexColorNumber.Substring(2, 2), 16);
                    int blue = Convert.ToInt32(hexColorNumber.Substring(4, 2), 16);
                    paletteColors[xi] = Color.FromArgb(red, green, blue);*/
                    double colorRelation = 255 * (((double)xi) / pictureBoxPalette.Size.Width);
                    int red = (int)colorRelation;
                    int green = (int)(255 - colorRelation);
                    int blue = 255;
                    paletteColors[xi] = Color.FromArgb(red, green, blue);
                }
                labelMin.Text = string.Format("{0:f3}", funcMin);
                labelMid.Text = string.Format("{0:f3}", funcMin + (funcMax - funcMin) / 2);
                labelMax.Text = string.Format("{0:f3}", funcMax);
                labelMin.Visible = true;
                labelMid.Visible = true;
                labelMax.Visible = true;
                isPaletteReady = true;
            }
            pictureBox1.Invalidate();
            pictureBoxPalette.Invalidate();
        }

        private void FormGraph_SizeChanged(object sender, EventArgs e)
        {
            InitializeFunction();
        }

        private void pictureBox1_Paint(object sender, PaintEventArgs e)
        {
            Cursor.Current = Cursors.WaitCursor;
            for (int xi = 0; xi < pictureBox1.Size.Width; xi++)
            {
                for (int yi = 0; yi < pictureBox1.Size.Height; yi++)
                {
                    e.Graphics.DrawRectangle(new Pen(colors[xi][yi]), xi, yi, 1, 1);
                }
            }
            Cursor.Current = Cursors.Default;
        }

        private void pictureBox1_MouseLeave(object sender, EventArgs e)
        {
            labelX.Visible = false;
            labelY.Visible = false;
            labelF.Visible = false;
        }

        private void pictureBox1_MouseMove(object sender, MouseEventArgs e)
        {
            labelX.Text = "X = " + (xmin + e.X * xStep);
            labelY.Text = "Y = " + (ymin + e.Y * yStep);
            labelF.Text = "F = " + funcValues[e.X][e.Y];
            labelX.Visible = true;
            labelY.Visible = true;
            labelF.Visible = true;
        }

        private void pictureBoxPalette_Paint(object sender, PaintEventArgs e)
        {
            for(int xi = 0; xi < pictureBoxPalette.Size.Width; xi++)
            {
                e.Graphics.DrawRectangle(new Pen(paletteColors[xi]), xi, 0, 1, pictureBoxPalette.Size.Height);
            }
        }
    }
}
