﻿/*
Ригин Антон Михайлович, ПИ ФКН НИУ ВШЭ, группа БПИ154.
Курсовая работа на 1-м курсе.
Тема курсовой работы: "Программа нейросетевого решения дифференциальных уравнений в частных производных".
Файл ArctgActivationFunction.cs библиотеки классов DiffEqNeuroLibrary, содержит класс ArctgActivationFunction, задающий функцию активации arctg.
*/

using System;
using AForge.Neuro; // Для представления структуры нейросетей используется AForge.NET Framework.

namespace DiffEqNeuroLibrary
{
    /// <summary>
    /// Задает функцию активации arctg.
    /// </summary>
    public class ArctgActivationFunction : IActivationFunction
    {
        /// <summary>
        /// Вычисляет значение функции активации arctg от аргумента x.
        /// </summary>
        /// <param name="x">Аргумент x.</param>
        /// <returns>Возвращает вычисленное значение функции активации arctg от аргумента x.</returns>
        public double Function(double x)
        {
            return Math.Atan(x);
        }

        /// <summary>
        /// Вычисляет производную функции активации arctg от аргумента x.
        /// </summary>
        /// <param name="x">Аргумент x.</param>
        /// <returns>Возвращает вычисленную производную функции активации arctg от аргумента x.</returns>
        public double Derivative(double x)
        {
            return 1 / (1 + x * x);
        }

        /// <summary>
        /// Вычисляет производную функции активации arctg от аргумента y.
        /// </summary>
        /// <param name="y">Аргумент y.</param>
        /// <returns>Возвращает вычисленную производную функции активации arctg от аргумента y.</returns>
        public double Derivative2(double y)
        {
            return 1 / (1 + y * y);
        }

        /// <summary>
        /// Вычисляет вторую производную функции активации arctg от аргумента x.
        /// </summary>
        /// <param name="x">Аргумент x.</param>
        /// <returns>Возвращает вычисленную вторую производную функции активации arctg от аргумента x.</returns>
        public double SecondDerivative(double x)
        {
            return -2 * x / ((1 + x * x) * (1 + x * x));
        }

        /// <summary>
        /// Вычисляет вторую производную функции активации arctg от аргумента y.
        /// </summary>
        /// <param name="y">Аргумент y.</param>
        /// <returns>Возвращает вычисленную вторую производную функции активации arctg от аргумента y.</returns>
        public double SecondDerivative2(double y)
        {
            return -2 * y / ((1 + y * y) * (1 + y * y));
        }

        /// <summary>
        /// Вычисляет третью производную функции активации arctg от аргумента x.
        /// </summary>
        /// <param name="x">Аргумент x.</param>
        /// <returns>Возвращает вычисленную третью производную функции активации arctg от аргумента x.</returns>
        public double ThirdDerivative(double x)
        {
            return -2 / ((1 + x * x) * (1 + x * x)) - 8 * x * x / ((1 + x * x) * (1 + x * x) * (1 + x * x));
        }

        /// <summary>
        /// Вычисляет третью производную функции активации arctg от аргумента y.
        /// </summary>
        /// <param name="y">Аргумент y.</param>
        /// <returns>Возвращает вычисленную третью производную функции активации arctg от аргумента y.</returns>
        public double ThirdDerivative2(double y)
        {
            return -2 / ((1 + y * y) * (1 + y * y)) - 8 * y * y / ((1 + y * y) * (1 + y * y) * (1 + y * y));
        }
    }
}
