﻿/*
Ригин Антон Михайлович, ПИ ФКН НИУ ВШЭ, группа БПИ154.
Курсовая работа на 1-м курсе.
Тема курсовой работы: "Программа нейросетевого решения дифференциальных уравнений в частных производных".
Файл NetDifferentiating.cs библиотеки классов DiffEqNeuroLibrary, содержит класс NetDifferentiating, представляющий статические методы для вычисления производных нейросети по входам и по весовым коэффициентам в заданной точке.
*/

using AForge.Neuro; // Для представления структуры нейросетей используется AForge.NET Framework.

namespace DiffEqNeuroLibrary
{
    /// <summary>
    /// Представляет статические методы для вычисления производных нейросети по входам и по весовым коэффициентам в заданной точке.
    /// </summary>
    public static class NetDifferentiating
    {
        /// <summary>
        /// Вычисляет производную f'x нейросети в заданной точке.
        /// </summary>
        /// <param name="net">Нейросеть.</param>
        /// <param name="x">Координата X заданной точки.</param>
        /// <param name="y">Координата Y заданной точки.</param>
        /// <returns>Возвращает вычисленную производную f'x нейросети в заданной точке.</returns>
        public static double NetDifferentX(ActivationNetwork net, double x, double y)
        {
            ArctgActivationFunction arctgFunc = new ArctgActivationFunction();
            return DifferentX(net, (ActivationNeuron)net.Layers[net.Layers.Length - 1].Neurons[0], arctgFunc, 1, x, y);
        }

        /// <summary>
        /// Вычисляет производную f'y нейросети в заданной точке.
        /// </summary>
        /// <param name="net">Нейросеть.</param>
        /// <param name="x">Координата X заданной точки.</param>
        /// <param name="y">Координата Y заданной точки.</param>
        /// <returns>Возвращает вычисленную производную f'y нейросети в заданной точке.</returns>
        public static double NetDifferentY(ActivationNetwork net, double x, double y)
        {
            ArctgActivationFunction arctgFunc = new ArctgActivationFunction();
            return DifferentY(net, (ActivationNeuron)net.Layers[net.Layers.Length - 1].Neurons[0], arctgFunc, 1, x, y);
        }

        /// <summary>
        /// Вычисляет вторую производную f''xx нейросети в заданной точке.
        /// </summary>
        /// <param name="net">Нейросеть.</param>
        /// <param name="x">Координата X заданной точки.</param>
        /// <param name="y">Координата Y заданной точки.</param>
        /// <returns>Возвращает вычисленную вторую производную f''xx нейросети в заданной точке.</returns>
        public static double NetDifferentXX(ActivationNetwork net, double x, double y)
        {
            ArctgActivationFunction arctgFunc = new ArctgActivationFunction();
            return DifferentXX(net, (ActivationNeuron)net.Layers[net.Layers.Length - 1].Neurons[0], arctgFunc, 1, x, y);
        }

        /// <summary>
        /// Вычисляет вторую производную f''xy нейросети в заданной точке.
        /// </summary>
        /// <param name="net">Нейросеть.</param>
        /// <param name="x">Координата X заданной точки.</param>
        /// <param name="y">Координата Y заданной точки.</param>
        /// <returns>Возвращает вычисленную вторую производную f''xy нейросети в заданной точке.</returns>
        public static double NetDifferentXY(ActivationNetwork net, double x, double y)
        {
            ArctgActivationFunction arctgFunc = new ArctgActivationFunction();
            return DifferentXY(net, (ActivationNeuron)net.Layers[net.Layers.Length - 1].Neurons[0], arctgFunc, 1, x, y);
        }

        /// <summary>
        /// Вычисляет вторую производную f''yx нейросети в заданной точке.
        /// </summary>
        /// <param name="net">Нейросеть.</param>
        /// <param name="x">Координата X заданной точки.</param>
        /// <param name="y">Координата Y заданной точки.</param>
        /// <returns>Возвращает вычисленную вторую производную f''yx нейросети в заданной точке.</returns>
        public static double NetDifferentYX(ActivationNetwork net, double x, double y)
        {
            ArctgActivationFunction arctgFunc = new ArctgActivationFunction();
            return DifferentYX(net, (ActivationNeuron)net.Layers[net.Layers.Length - 1].Neurons[0], arctgFunc, 1, x, y);
        }

        /// <summary>
        /// Вычисляет вторую производную f''yy нейросети в заданной точке.
        /// </summary>
        /// <param name="net">Нейросеть.</param>
        /// <param name="x">Координата X заданной точки.</param>
        /// <param name="y">Координата Y заданной точки.</param>
        /// <returns>Возвращает вычисленную вторую производную f''yy нейросети в заданной точке.</returns>
        public static double NetDifferentYY(ActivationNetwork net, double x, double y)
        {
            ArctgActivationFunction arctgFunc = new ArctgActivationFunction();
            return DifferentYY(net, (ActivationNeuron)net.Layers[net.Layers.Length - 1].Neurons[0], arctgFunc, 1, x, y);
        }

        /// <summary>
        /// Вычисляет производную нейросети по заданному весовому коэффициенту в заданной точке.
        /// </summary>
        /// <param name="net">Нейросеть.</param>
        /// <param name="neuronOutputs">Выходы нейронов нейросети в заданной точке.</param>
        /// <param name="activFuncDerivatives">Производные функции активации arctg от выходов нейронов нейросети в заданной точке.</param>
        /// <param name="isActivFuncDerivativeReady">Показывает, какие производные функции активации arctg от выходов нейронов нейросети в заданной точке уже посчитаны.</param>
        /// <param name="layerNumber">Номер слоя заданного весового коэффициента.</param>
        /// <param name="neuronInLayerNumber">Номер нейрона заданного весового коэффициента в слое.</param>
        /// <param name="weightInNeuronNumber">Номер заданного весового коэффициента в нейроне.</param>
        /// <param name="x">Координата X заданной точки.</param>
        /// <param name="y">Координата Y заданной точки.</param>
        /// <returns>Возвращает вычисленную производную нейросети по заданному весовому коэффициенту в заданной точке.</returns>
        public static double NetDifferentWeight(ActivationNetwork net, double[][] neuronOutputs, ref double[][] activFuncDerivatives, ref bool[][] isActivFuncDerivativeReady, int layerNumber, int neuronInLayerNumber, int weightInNeuronNumber, double x, double y)
        {
            ArctgActivationFunction arctgFunc = new ArctgActivationFunction();
            return DifferentWeight(net, (ActivationNeuron)net.Layers[net.Layers.Length - 1].Neurons[0], arctgFunc, neuronOutputs, net.Layers.Length - 1, 0, ref activFuncDerivatives, ref isActivFuncDerivativeReady, layerNumber, neuronInLayerNumber, weightInNeuronNumber, x, y);
        }

        /// <summary>
        /// Вычисляет производную производной f'x нейросети по заданному весовому коэффициенту в заданной точке.
        /// </summary>
        /// <param name="net">Нейросеть.</param>
        /// <param name="neuronOutputs">Выходы нейронов нейросети в заданной точке.</param>
        /// <param name="differentsX">Производные f'x выходов нейронов нейросети в заданной точке.</param>
        /// <param name="activFuncDerivatives">Производные функции активации arctg от выходов нейронов нейросети в заданной точке.</param>
        /// <param name="isActivFuncDerivativeReady">Показывает, какие производные функции активации arctg от выходов нейронов нейросети в заданной точке уже посчитаны.</param>
        /// <param name="activFuncSecondDerivatives">Вторые производные функции активации arctg от выходов нейронов нейросети в заданной точке.</param>
        /// <param name="isActivFuncSecondDerivativeReady">Показывает, какие вторые производные функции активации arctg от выходов нейронов нейросети в заданной точке уже посчитаны.</param>
        /// <param name="layerNumber">Номер слоя заданного весового коэффициента.</param>
        /// <param name="neuronInLayerNumber">Номер нейрона заданного весового коэффициента в слое.</param>
        /// <param name="weightInNeuronNumber">Номер заданного весового коэффициента в нейроне.</param>
        /// <param name="x">Координата X заданной точки.</param>
        /// <param name="y">Координата Y заданной точки.</param>
        /// <returns>Возвращает вычисленную производную производной f'x нейросети по заданному весовому коэффициенту в заданной точке.</returns>
        public static double NetDifferentXWeight(ActivationNetwork net, double[][] neuronOutputs, double[][] differentsX, ref double[][] activFuncDerivatives, ref bool[][] isActivFuncDerivativeReady, ref double[][] activFuncSecondDerivatives, ref bool[][] isActivFuncSecondDerivativeReady, int layerNumber, int neuronInLayerNumber, int weightInNeuronNumber, double x, double y)
        {
            ArctgActivationFunction arctgFunc = new ArctgActivationFunction();
            return DifferentXWeight(net, (ActivationNeuron)net.Layers[net.Layers.Length - 1].Neurons[0], arctgFunc, neuronOutputs, differentsX, net.Layers.Length - 1, 0, ref activFuncDerivatives, ref isActivFuncDerivativeReady, ref activFuncSecondDerivatives, ref isActivFuncSecondDerivativeReady, layerNumber, neuronInLayerNumber, weightInNeuronNumber, x, y);
        }

        /// <summary>
        /// Вычисляет производную производной f'y нейросети по заданному весовому коэффициенту в заданной точке.
        /// </summary>
        /// <param name="net">Нейросеть.</param>
        /// <param name="neuronOutputs">Выходы нейронов нейросети в заданной точке.</param>
        /// <param name="differentsY">Производные f'y выходов нейронов нейросети в заданной точке.</param>
        /// <param name="activFuncDerivatives">Производные функции активации arctg от выходов нейронов нейросети в заданной точке.</param>
        /// <param name="isActivFuncDerivativeReady">Показывает, какие производные функции активации arctg от выходов нейронов нейросети в заданной точке уже посчитаны.</param>
        /// <param name="activFuncSecondDerivatives">Вторые производные функции активации arctg от выходов нейронов нейросети в заданной точке.</param>
        /// <param name="isActivFuncSecondDerivativeReady">Показывает, какие вторые производные функции активации arctg от выходов нейронов нейросети в заданной точке уже посчитаны.</param>
        /// <param name="layerNumber">Номер слоя заданного весового коэффициента.</param>
        /// <param name="neuronInLayerNumber">Номер нейрона заданного весового коэффициента в слое.</param>
        /// <param name="weightInNeuronNumber">Номер заданного весового коэффициента в нейроне.</param>
        /// <param name="x">Координата X заданной точки.</param>
        /// <param name="y">Координата Y заданной точки.</param>
        /// <returns>Возвращает вычисленную производную производной f'x нейросети по заданному весовому коэффициенту в заданной точке.</returns>
        public static double NetDifferentYWeight(ActivationNetwork net, double[][] neuronOutputs, double[][] differentsY, ref double[][] activFuncDerivatives, ref bool[][] isActivFuncDerivativeReady, ref double[][] activFuncSecondDerivatives, ref bool[][] isActivFuncSecondDerivativeReady, int layerNumber, int neuronInLayerNumber, int weightInNeuronNumber, double x, double y)
        {
            ArctgActivationFunction arctgFunc = new ArctgActivationFunction();
            return DifferentYWeight(net, (ActivationNeuron)net.Layers[net.Layers.Length - 1].Neurons[0], arctgFunc, neuronOutputs, differentsY, net.Layers.Length - 1, 0, ref activFuncDerivatives, ref isActivFuncDerivativeReady, ref activFuncSecondDerivatives, ref isActivFuncSecondDerivativeReady, layerNumber, neuronInLayerNumber, weightInNeuronNumber, x, y);
        }

        /// <summary>
        /// Вычисляет производную второй производной f''xx нейросети по заданному весовому коэффициенту в заданной точке.
        /// </summary>
        /// <param name="net">Нейросеть.</param>
        /// <param name="neuronOutputs">Выходы нейронов нейросети в заданной точке.</param>
        /// <param name="differentsX">Производные f'x выходов нейронов нейросети в заданной точке.</param>
        /// <param name="differentsXX">Вторые производные f''xx выходов нейронов нейросети в заданной точке.</param>
        /// <param name="activFuncDerivatives">Производные функции активации arctg от выходов нейронов нейросети в заданной точке.</param>
        /// <param name="isActivFuncDerivativeReady">Показывает, какие производные функции активации arctg от выходов нейронов нейросети в заданной точке уже посчитаны.</param>
        /// <param name="activFuncSecondDerivatives">Вторые производные функции активации arctg от выходов нейронов нейросети в заданной точке.</param>
        /// <param name="isActivFuncSecondDerivativeReady">Показывает, какие вторые производные функции активации arctg от выходов нейронов нейросети в заданной точке уже посчитаны.</param>
        /// <param name="activFuncThirdDerivatives">Третьи производные функции активации arctg от выходов нейронов нейросети в заданной точке.</param>
        /// <param name="isActivFuncThirdDerivativeReady">Показывает, какие третьи производные функции активации arctg от выходов нейронов нейросети в заданной точке уже посчитаны.</param>
        /// <param name="layerNumber">Номер слоя заданного весового коэффициента.</param>
        /// <param name="neuronInLayerNumber">Номер нейрона заданного весового коэффициента в слое.</param>
        /// <param name="weightInNeuronNumber">Номер заданного весового коэффициента в нейроне.</param>
        /// <param name="x">Координата X заданной точки.</param>
        /// <param name="y">Координата Y заданной точки.</param>
        /// <returns>Возвращает вычисленную производную второй производной f''xx нейросети по заданному весовому коэффициенту в заданной точке.</returns>
        public static double NetDifferentXXWeight(ActivationNetwork net, double[][] neuronOutputs, double[][] differentsX, double[][] differentsXX, ref double[][] activFuncDerivatives, ref bool[][] isActivFuncDerivativeReady, ref double[][] activFuncSecondDerivatives, ref bool[][] isActivFuncSecondDerivativeReady, ref double[][] activFuncThirdDerivatives, ref bool[][] isActivFuncThirdDerivativeReady, int layerNumber, int neuronInLayerNumber, int weightInNeuronNumber, double x, double y)
        {
            ArctgActivationFunction arctgFunc = new ArctgActivationFunction();
            return DifferentXXWeight(net, (ActivationNeuron)net.Layers[net.Layers.Length - 1].Neurons[0], arctgFunc, neuronOutputs, differentsX, differentsXX, net.Layers.Length - 1, 0, ref activFuncDerivatives, ref isActivFuncDerivativeReady, ref activFuncSecondDerivatives, ref isActivFuncSecondDerivativeReady, ref activFuncThirdDerivatives, ref isActivFuncThirdDerivativeReady, layerNumber, neuronInLayerNumber, weightInNeuronNumber, x, y);
        }

        /// <summary>
        /// Вычисляет производную второй производной f''xy нейросети по заданному весовому коэффициенту в заданной точке.
        /// </summary>
        /// <param name="net">Нейросеть.</param>
        /// <param name="neuronOutputs">Выходы нейронов нейросети в заданной точке.</param>
        /// <param name="differentsX">Производные f'x выходов нейронов нейросети в заданной точке.</param>
        /// <param name="differentsXY">Вторые производные f''xy выходов нейронов нейросети в заданной точке.</param>
        /// <param name="activFuncDerivatives">Производные функции активации arctg от выходов нейронов нейросети в заданной точке.</param>
        /// <param name="isActivFuncDerivativeReady">Показывает, какие производные функции активации arctg от выходов нейронов нейросети в заданной точке уже посчитаны.</param>
        /// <param name="activFuncSecondDerivatives">Вторые производные функции активации arctg от выходов нейронов нейросети в заданной точке.</param>
        /// <param name="isActivFuncSecondDerivativeReady">Показывает, какие вторые производные функции активации arctg от выходов нейронов нейросети в заданной точке уже посчитаны.</param>
        /// <param name="activFuncThirdDerivatives">Третьи производные функции активации arctg от выходов нейронов нейросети в заданной точке.</param>
        /// <param name="isActivFuncThirdDerivativeReady">Показывает, какие третьи производные функции активации arctg от выходов нейронов нейросети в заданной точке уже посчитаны.</param>
        /// <param name="layerNumber">Номер слоя заданного весового коэффициента.</param>
        /// <param name="neuronInLayerNumber">Номер нейрона заданного весового коэффициента в слое.</param>
        /// <param name="weightInNeuronNumber">Номер заданного весового коэффициента в нейроне.</param>
        /// <param name="x">Координата X заданной точки.</param>
        /// <param name="y">Координата Y заданной точки.</param>
        /// <returns>Возвращает вычисленную производную второй производной f''xy нейросети по заданному весовому коэффициенту в заданной точке.</returns>
        public static double NetDifferentXYWeight(ActivationNetwork net, double[][] neuronOutputs, double[][] differentsX, double[][] differentsXY, ref double[][] activFuncDerivatives, ref bool[][] isActivFuncDerivativeReady, ref double[][] activFuncSecondDerivatives, ref bool[][] isActivFuncSecondDerivativeReady, ref double[][] activFuncThirdDerivatives, ref bool[][] isActivFuncThirdDerivativeReady, int layerNumber, int neuronInLayerNumber, int weightInNeuronNumber, double x, double y)
        {
            ArctgActivationFunction arctgFunc = new ArctgActivationFunction();
            return DifferentXYWeight(net, (ActivationNeuron)net.Layers[net.Layers.Length - 1].Neurons[0], arctgFunc, neuronOutputs, differentsX, differentsXY, net.Layers.Length - 1, 0, ref activFuncDerivatives, ref isActivFuncDerivativeReady, ref activFuncSecondDerivatives, ref isActivFuncSecondDerivativeReady, ref activFuncThirdDerivatives, ref isActivFuncThirdDerivativeReady, layerNumber, neuronInLayerNumber, weightInNeuronNumber, x, y);
        }

        /// <summary>
        /// Вычисляет производную второй производной f''yx нейросети по заданному весовому коэффициенту в заданной точке.
        /// </summary>
        /// <param name="net">Нейросеть.</param>
        /// <param name="neuronOutputs">Выходы нейронов нейросети в заданной точке.</param>
        /// <param name="differentsY">Производные f'y выходов нейронов нейросети в заданной точке.</param>
        /// <param name="differentsYX">Вторые производные f''yx выходов нейронов нейросети в заданной точке.</param>
        /// <param name="activFuncDerivatives">Производные функции активации arctg от выходов нейронов нейросети в заданной точке.</param>
        /// <param name="isActivFuncDerivativeReady">Показывает, какие производные функции активации arctg от выходов нейронов нейросети в заданной точке уже посчитаны.</param>
        /// <param name="activFuncSecondDerivatives">Вторые производные функции активации arctg от выходов нейронов нейросети в заданной точке.</param>
        /// <param name="isActivFuncSecondDerivativeReady">Показывает, какие вторые производные функции активации arctg от выходов нейронов нейросети в заданной точке уже посчитаны.</param>
        /// <param name="activFuncThirdDerivatives">Третьи производные функции активации arctg от выходов нейронов нейросети в заданной точке.</param>
        /// <param name="isActivFuncThirdDerivativeReady">Показывает, какие третьи производные функции активации arctg от выходов нейронов нейросети в заданной точке уже посчитаны.</param>
        /// <param name="layerNumber">Номер слоя заданного весового коэффициента.</param>
        /// <param name="neuronInLayerNumber">Номер нейрона заданного весового коэффициента в слое.</param>
        /// <param name="weightInNeuronNumber">Номер заданного весового коэффициента в нейроне.</param>
        /// <param name="x">Координата X заданной точки.</param>
        /// <param name="y">Координата Y заданной точки.</param>
        /// <returns>Возвращает вычисленную производную второй производной f''yx нейросети по заданному весовому коэффициенту в заданной точке.</returns>
        public static double NetDifferentYXWeight(ActivationNetwork net, double[][] neuronOutputs, double[][] differentsY, double[][] differentsYX, ref double[][] activFuncDerivatives, ref bool[][] isActivFuncDerivativeReady, ref double[][] activFuncSecondDerivatives, ref bool[][] isActivFuncSecondDerivativeReady, ref double[][] activFuncThirdDerivatives, ref bool[][] isActivFuncThirdDerivativeReady, int layerNumber, int neuronInLayerNumber, int weightInNeuronNumber, double x, double y)
        {
            ArctgActivationFunction arctgFunc = new ArctgActivationFunction();
            return DifferentYXWeight(net, (ActivationNeuron)net.Layers[net.Layers.Length - 1].Neurons[0], arctgFunc, neuronOutputs, differentsY, differentsYX, net.Layers.Length - 1, 0, ref activFuncDerivatives, ref isActivFuncDerivativeReady, ref activFuncSecondDerivatives, ref isActivFuncSecondDerivativeReady, ref activFuncThirdDerivatives, ref isActivFuncThirdDerivativeReady, layerNumber, neuronInLayerNumber, weightInNeuronNumber, x, y);
        }

        /// <summary>
        /// Вычисляет производную второй производной f''yy нейросети по заданному весовому коэффициенту в заданной точке.
        /// </summary>
        /// <param name="net">Нейросеть.</param>
        /// <param name="neuronOutputs">Выходы нейронов нейросети в заданной точке.</param>
        /// <param name="differentsY">Производные f'y выходов нейронов нейросети в заданной точке.</param>
        /// <param name="differentsYX">Вторые производные f''yy выходов нейронов нейросети в заданной точке.</param>
        /// <param name="activFuncDerivatives">Производные функции активации arctg от выходов нейронов нейросети в заданной точке.</param>
        /// <param name="isActivFuncDerivativeReady">Показывает, какие производные функции активации arctg от выходов нейронов нейросети в заданной точке уже посчитаны.</param>
        /// <param name="activFuncSecondDerivatives">Вторые производные функции активации arctg от выходов нейронов нейросети в заданной точке.</param>
        /// <param name="isActivFuncSecondDerivativeReady">Показывает, какие вторые производные функции активации arctg от выходов нейронов нейросети в заданной точке уже посчитаны.</param>
        /// <param name="activFuncThirdDerivatives">Третьи производные функции активации arctg от выходов нейронов нейросети в заданной точке.</param>
        /// <param name="isActivFuncThirdDerivativeReady">Показывает, какие третьи производные функции активации arctg от выходов нейронов нейросети в заданной точке уже посчитаны.</param>
        /// <param name="layerNumber">Номер слоя заданного весового коэффициента.</param>
        /// <param name="neuronInLayerNumber">Номер нейрона заданного весового коэффициента в слое.</param>
        /// <param name="weightInNeuronNumber">Номер заданного весового коэффициента в нейроне.</param>
        /// <param name="x">Координата X заданной точки.</param>
        /// <param name="y">Координата Y заданной точки.</param>
        /// <returns>Возвращает вычисленную производную второй производной f''yy нейросети по заданному весовому коэффициенту в заданной точке.</returns>
        public static double NetDifferentYYWeight(ActivationNetwork net, double[][] neuronOutputs, double[][] differentsY, double[][] differentsYY, ref double[][] activFuncDerivatives, ref bool[][] isActivFuncDerivativeReady, ref double[][] activFuncSecondDerivatives, ref bool[][] isActivFuncSecondDerivativeReady, ref double[][] activFuncThirdDerivatives, ref bool[][] isActivFuncThirdDerivativeReady, int layerNumber, int neuronInLayerNumber, int weightInNeuronNumber, double x, double y)
        {
            ArctgActivationFunction arctgFunc = new ArctgActivationFunction();
            return DifferentYYWeight(net, (ActivationNeuron)net.Layers[net.Layers.Length - 1].Neurons[0], arctgFunc, neuronOutputs, differentsY, differentsYY, net.Layers.Length - 1, 0, ref activFuncDerivatives, ref isActivFuncDerivativeReady, ref activFuncSecondDerivatives, ref isActivFuncSecondDerivativeReady, ref activFuncThirdDerivatives, ref isActivFuncThirdDerivativeReady, layerNumber, neuronInLayerNumber, weightInNeuronNumber, x, y);
        }

        /// <summary>
        /// Вычисляет выход заданного нейрона нейросети в заданной точке.
        /// </summary>
        /// <param name="net">Нейросеть.</param>
        /// <param name="outputNeuron">Заданный нейрон нейросети.</param>
        /// <param name="depth">Текущая "глубина" рекурсии.</param>
        /// <param name="x">Координата X заданной точки.</param>
        /// <param name="y">Координата Y заданной точки.</param>
        /// <returns>Возвращает вычисленный выход заданного нейрона нейросети в заданной точке.</returns>
        public static double ComputeNeuronOutput(ActivationNetwork net, Neuron outputNeuron, int depth, double x, double y)
        {
            if (depth == net.Layers.Length)
            {
                double[] input = { x, y };
                return outputNeuron.Compute(input);
            }
            else
            {
                double[] input = new double[outputNeuron.InputsCount];
                for (int i = 0; i < input.Length; i++)
                {
                    input[i] = ComputeNeuronOutput(net, net.Layers[net.Layers.Length - depth - 1].Neurons[i], depth + 1, x, y);
                }
                return outputNeuron.Compute(input);
            }
        }

        /// <summary>
        /// Вычисляет выходы нейронов нейросети в заданной точке.
        /// </summary>
        /// <param name="net">Нейросеть.</param>
        /// <param name="x">Координата X заданной точки.</param>
        /// <param name="y">Координата Y заданной точки.</param>
        /// <returns>Возвращает вычисленные выходы нейронов нейросети в заданной точке.</returns>
        public static double[][] ComputeNeuronOutputs(ActivationNetwork net, double x, double y)
        {
            double[][] neuronOutputs = new double[net.Layers.Length][];
            neuronOutputs[0] = new double[net.Layers[0].Neurons.Length];
            for (int j = 0; j < net.Layers[0].Neurons.Length; j++)
            {
                double[] input = { x, y };
                neuronOutputs[0][j] = net.Layers[0].Neurons[j].Compute(input);
            }
            for (int i = 1; i < net.Layers.Length; i++)
            {
                neuronOutputs[i] = new double[net.Layers[i].Neurons.Length];
                for (int j = 0; j < net.Layers[i].Neurons.Length; j++)
                {
                    neuronOutputs[i][j] = net.Layers[i].Neurons[j].Compute(neuronOutputs[i - 1]);
                }
            }
            return neuronOutputs;
        }

        /// <summary>
        /// Вычисляет производную f'x заданного нейрона нейросети в заданной точке.
        /// </summary>
        /// <param name="net">Нейросеть.</param>
        /// <param name="outputNeuron">Заданный нейрон нейросети.</param>
        /// <param name="arctgFunc">Функция активации arctg.</param>
        /// <param name="depth">Текущая "глубина" рекурсии.</param>
        /// <param name="x">Координата X заданной точки.</param>
        /// <param name="y">Координата Y заданной точки.</param>
        /// <returns>Возвращает вычисленную производную f'x заданного нейрона нейросети в заданной точке.</returns>
        static double DifferentX(ActivationNetwork net, ActivationNeuron outputNeuron, ArctgActivationFunction arctgFunc, int depth, double x, double y)
        {
            if (depth == net.Layers.Length)
            {
                return arctgFunc.Derivative(outputNeuron.Weights[0] * x + outputNeuron.Weights[1] * y + outputNeuron.Threshold) * outputNeuron.Weights[0];
            }
            else
            {
                double innerDifferent = 0;
                for (int i = 0; i < outputNeuron.InputsCount; i++)
                {
                    innerDifferent += outputNeuron.Weights[i] * DifferentX(net, (ActivationNeuron)net.Layers[net.Layers.Length - depth - 1].Neurons[i], arctgFunc, depth + 1, x, y);
                }
                if (depth == 1)
                {
                    return innerDifferent;
                }
                else
                {
                    double O = 0;
                    for (int i = 0; i < outputNeuron.InputsCount; i++)
                    {
                        O += outputNeuron.Weights[i] * ComputeNeuronOutput(net, net.Layers[net.Layers.Length - depth - 1].Neurons[i], depth + 1, x, y);
                    }
                    O += outputNeuron.Threshold;
                    return arctgFunc.Derivative(O) * innerDifferent;
                }
            }
        }

        /// <summary>
        /// Вычисляет производные f'x нейронов нейросети в заданной точке.
        /// </summary>
        /// <param name="net">Нейросеть.</param>
        /// <param name="arctgFunc">Функция активации arctg.</param>
        /// <param name="neuronOutputs">Выходы нейронов нейросети в заданной точке.</param>
        /// <param name="activFuncDerivatives">Производные функции активации arctg от выходов нейронов нейросети в заданной точке.</param>
        /// <param name="isActivFuncDerivativeReady">Показывает, какие производные функции активации arctg от выходов нейронов нейросети в заданной точке уже посчитаны.</param>
        /// <param name="x">Координата X заданной точки.</param>
        /// <param name="y">Координата Y заданной точки.</param>
        /// <returns>Возвращает вычисленные производные f'x нейронов нейросети в заданной точке.</returns>
        public static double[][] DifferentsX(ActivationNetwork net, ArctgActivationFunction arctgFunc, double[][] neuronOutputs, ref double[][] activFuncDerivatives, ref bool[][] isActivFuncDerivativeReady, double x, double y)
        {
            double[][] differentsX = new double[net.Layers.Length][];
            differentsX[0] = new double[net.Layers[0].Neurons.Length];
            for (int j = 0; j < net.Layers[0].Neurons.Length; j++)
            {
                if(!isActivFuncDerivativeReady[0][j])
                {
                    activFuncDerivatives[0][j] = arctgFunc.Derivative(net.Layers[0].Neurons[j].Weights[0] * x + net.Layers[0].Neurons[j].Weights[1] * y + ((ActivationNeuron)(net.Layers[0].Neurons[j])).Threshold);
                    isActivFuncDerivativeReady[0][j] = true;
                }
                differentsX[0][j] = activFuncDerivatives[0][j] * net.Layers[0].Neurons[j].Weights[0];
            }
            for (int i = 1; i < net.Layers.Length - 1; i++)
            {
                differentsX[i] = new double[net.Layers[i].Neurons.Length];
                for (int j = 0; j < net.Layers[i].Neurons.Length; j++)
                {
                    double innerDifferent = 0;
                    for (int k = 0; k < net.Layers[i].Neurons[j].InputsCount; k++)
                    {
                        innerDifferent += net.Layers[i].Neurons[j].Weights[k] * differentsX[i - 1][k];
                    }
                    if(!isActivFuncDerivativeReady[i][j])
                    {
                        double O = 0;
                        for (int k = 0; k < net.Layers[i].Neurons[j].InputsCount; k++)
                        {
                            O += net.Layers[i].Neurons[j].Weights[k] * neuronOutputs[i - 1][k];
                        }
                        O += ((ActivationNeuron)(net.Layers[i].Neurons[j])).Threshold;
                        activFuncDerivatives[i][j] = arctgFunc.Derivative(O);
                        isActivFuncDerivativeReady[i][j] = true;
                    }
                    differentsX[i][j] = activFuncDerivatives[i][j] * innerDifferent;
                }
            }
            differentsX[net.Layers.Length - 1] = new double[net.Layers[net.Layers.Length - 1].Neurons.Length];
            for (int j = 0; j < net.Layers[net.Layers.Length - 1].Neurons.Length; j++)
            {
                double innerDifferent = 0;
                for (int k = 0; k < net.Layers[net.Layers.Length - 1].Neurons[j].InputsCount; k++)
                {
                    innerDifferent += net.Layers[net.Layers.Length - 1].Neurons[j].Weights[k] * differentsX[net.Layers.Length - 2][k];
                }
                differentsX[net.Layers.Length - 1][j] = innerDifferent;
            }
            return differentsX;
        }

        /// <summary>
        /// Вычисляет производную f'y заданного нейрона нейросети в заданной точке.
        /// </summary>
        /// <param name="net">Нейросеть.</param>
        /// <param name="outputNeuron">Заданный нейрон нейросети.</param>
        /// <param name="arctgFunc">Функция активации arctg.</param>
        /// <param name="depth">Текущая "глубина" рекурсии.</param>
        /// <param name="x">Координата X заданной точки.</param>
        /// <param name="y">Координата Y заданной точки.</param>
        /// <returns>Возвращает вычисленную производную f'y заданного нейрона нейросети в заданной точке.</returns>
        static double DifferentY(ActivationNetwork net, ActivationNeuron outputNeuron, ArctgActivationFunction arctgFunc, int depth, double x, double y)
        {
            if (depth == net.Layers.Length)
            {
                return arctgFunc.Derivative(outputNeuron.Weights[0] * x + outputNeuron.Weights[1] * y + outputNeuron.Threshold) * outputNeuron.Weights[1];
            }
            else
            {
                double innerDifferent = 0;
                for (int i = 0; i < outputNeuron.InputsCount; i++)
                {
                    innerDifferent += outputNeuron.Weights[i] * DifferentY(net, (ActivationNeuron)net.Layers[net.Layers.Length - depth - 1].Neurons[i], arctgFunc, depth + 1, x, y);
                }
                if (depth == 1)
                {
                    return innerDifferent;
                }
                else
                {
                    double O = 0;
                    for (int i = 0; i < outputNeuron.InputsCount; i++)
                    {
                        O += outputNeuron.Weights[i] * ComputeNeuronOutput(net, net.Layers[net.Layers.Length - depth - 1].Neurons[i], depth + 1, x, y);
                    }
                    O += outputNeuron.Threshold;
                    return arctgFunc.Derivative(O) * innerDifferent;
                }
            }
        }

        /// <summary>
        /// Вычисляет производные f'y нейронов нейросети в заданной точке.
        /// </summary>
        /// <param name="net">Нейросеть.</param>
        /// <param name="arctgFunc">Функция активации arctg.</param>
        /// <param name="neuronOutputs">Выходы нейронов нейросети в заданной точке.</param>
        /// <param name="activFuncDerivatives">Производные функции активации arctg от выходов нейронов нейросети в заданной точке.</param>
        /// <param name="isActivFuncDerivativeReady">Показывает, какие производные функции активации arctg от выходов нейронов нейросети в заданной точке уже посчитаны.</param>
        /// <param name="x">Координата X заданной точки.</param>
        /// <param name="y">Координата Y заданной точки.</param>
        /// <returns>Возвращает вычисленные производные f'y нейронов нейросети в заданной точке.</returns>
        public static double[][] DifferentsY(ActivationNetwork net, ArctgActivationFunction arctgFunc, double[][] neuronOutputs, ref double[][] activFuncDerivatives, ref bool[][] isActivFuncDerivativeReady, double x, double y)
        {
            double[][] differentsY = new double[net.Layers.Length][];
            differentsY[0] = new double[net.Layers[0].Neurons.Length];
            for (int j = 0; j < net.Layers[0].Neurons.Length; j++)
            {
                if (!isActivFuncDerivativeReady[0][j])
                {
                    activFuncDerivatives[0][j] = arctgFunc.Derivative(net.Layers[0].Neurons[j].Weights[0] * x + net.Layers[0].Neurons[j].Weights[1] * y + ((ActivationNeuron)(net.Layers[0].Neurons[j])).Threshold);
                    isActivFuncDerivativeReady[0][j] = true;
                }
                differentsY[0][j] = activFuncDerivatives[0][j] * net.Layers[0].Neurons[j].Weights[1];
            }
            for (int i = 1; i < net.Layers.Length - 1; i++)
            {
                differentsY[i] = new double[net.Layers[i].Neurons.Length];
                for (int j = 0; j < net.Layers[i].Neurons.Length; j++)
                {
                    double innerDifferent = 0;
                    for (int k = 0; k < net.Layers[i].Neurons[j].InputsCount; k++)
                    {
                        innerDifferent += net.Layers[i].Neurons[j].Weights[k] * differentsY[i - 1][k];
                    }
                    if (!isActivFuncDerivativeReady[i][j])
                    {
                        double O = 0;
                        for (int k = 0; k < net.Layers[i].Neurons[j].InputsCount; k++)
                        {
                            O += net.Layers[i].Neurons[j].Weights[k] * neuronOutputs[i - 1][k];
                        }
                        O += ((ActivationNeuron)(net.Layers[i].Neurons[j])).Threshold;
                        activFuncDerivatives[i][j] = arctgFunc.Derivative(O);
                        isActivFuncDerivativeReady[i][j] = true;
                    }
                    differentsY[i][j] = activFuncDerivatives[i][j] * innerDifferent;
                }
            }
            differentsY[net.Layers.Length - 1] = new double[net.Layers[net.Layers.Length - 1].Neurons.Length];
            for (int j = 0; j < net.Layers[net.Layers.Length - 1].Neurons.Length; j++)
            {
                double innerDifferent = 0;
                for (int k = 0; k < net.Layers[net.Layers.Length - 1].Neurons[j].InputsCount; k++)
                {
                    innerDifferent += net.Layers[net.Layers.Length - 1].Neurons[j].Weights[k] * differentsY[net.Layers.Length - 2][k];
                }
                differentsY[net.Layers.Length - 1][j] = innerDifferent;
            }
            return differentsY;
        }

        /// <summary>
        /// Вычисляет вторую производную f''xx заданного нейрона нейросети в заданной точке.
        /// </summary>
        /// <param name="net">Нейросеть.</param>
        /// <param name="outputNeuron">Заданный нейрон нейросети.</param>
        /// <param name="arctgFunc">Функция активации arctg.</param>
        /// <param name="depth">Текущая "глубина" рекурсии.</param>
        /// <param name="x">Координата X заданной точки.</param>
        /// <param name="y">Координата Y заданной точки.</param>
        /// <returns>Возвращает вычисленную вторую производную f''xx заданного нейрона нейросети в заданной точке.</returns>
        static double DifferentXX(ActivationNetwork net, ActivationNeuron outputNeuron, ArctgActivationFunction arctgFunc, int depth, double x, double y)
        {
            if (depth == net.Layers.Length)
            {
                return arctgFunc.SecondDerivative(outputNeuron.Weights[0] * x + outputNeuron.Weights[1] * y + outputNeuron.Threshold) * outputNeuron.Weights[0] * outputNeuron.Weights[0];
            }
            else
            {
                double innerSecondDifferent = 0;
                for(int i = 0; i < outputNeuron.InputsCount; i++)
                {
                    innerSecondDifferent += outputNeuron.Weights[i] * DifferentXX(net, (ActivationNeuron)net.Layers[net.Layers.Length - depth - 1].Neurons[i], arctgFunc, depth + 1, x, y);
                }
                if(depth == 1)
                {
                    return innerSecondDifferent;
                }
                else
                {
                    double innerDifferent = 0;
                    for (int i = 0; i < outputNeuron.InputsCount; i++)
                    {
                        innerDifferent += outputNeuron.Weights[i] * DifferentX(net, (ActivationNeuron)net.Layers[net.Layers.Length - depth - 1].Neurons[i], arctgFunc, depth + 1, x, y);
                    }
                    double O = 0;
                    for (int i = 0; i < outputNeuron.InputsCount; i++)
                    {
                        O += outputNeuron.Weights[i] * ComputeNeuronOutput(net, net.Layers[net.Layers.Length - depth - 1].Neurons[i], depth + 1, x, y);
                    }
                    O += outputNeuron.Threshold;
                    return arctgFunc.SecondDerivative(O) * innerDifferent * innerDifferent + arctgFunc.Derivative(O) * innerSecondDifferent;
                }
            }
        }

        /// <summary>
        /// Вычисляет вторые производные f''xx нейронов нейросети в заданной точке.
        /// </summary>
        /// <param name="net">Нейросеть.</param>
        /// <param name="arctgFunc">Функция активации arctg.</param>
        /// <param name="neuronOutputs">Выходы нейронов нейросети в заданной точке.</param>
        /// <param name="differentsX">Производные f'x выходов нейронов нейросети в заданной точке.</param>
        /// <param name="activFuncDerivatives">Производные функции активации arctg от выходов нейронов нейросети в заданной точке.</param>
        /// <param name="isActivFuncDerivativeReady">Показывает, какие производные функции активации arctg от выходов нейронов нейросети в заданной точке уже посчитаны.</param>
        /// <param name="activFuncSecondDerivatives">Вторые производные функции активации arctg от выходов нейронов нейросети в заданной точке.</param>
        /// <param name="isActivFuncSecondDerivativeReady">Показывает, какие вторые производные функции активации arctg от выходов нейронов нейросети в заданной точке уже посчитаны.</param>
        /// <param name="x">Координата X заданной точки.</param>
        /// <param name="y">Координата Y заданной точки.</param>
        /// <returns>Возвращает вычисленные вторые производные f''xx нейронов нейросети в заданной точке.</returns>
        public static double[][] DifferentsXX(ActivationNetwork net, ArctgActivationFunction arctgFunc, double[][] neuronOutputs, double[][] differentsX, ref double[][] activFuncDerivatives, ref bool[][] isActivFuncDerivativeReady, ref double[][] activFuncSecondDerivatives, ref bool[][] isActivFuncSecondDerivativeReady, double x, double y)
        {
            double[][] differentsXX = new double[net.Layers.Length][];
            differentsXX[0] = new double[net.Layers[0].Neurons.Length];
            for (int j = 0; j < net.Layers[0].Neurons.Length; j++)
            {
                if (!isActivFuncSecondDerivativeReady[0][j])
                {
                    activFuncSecondDerivatives[0][j] = arctgFunc.SecondDerivative(net.Layers[0].Neurons[j].Weights[0] * x + net.Layers[0].Neurons[j].Weights[1] * y + ((ActivationNeuron)(net.Layers[0].Neurons[j])).Threshold);
                    isActivFuncSecondDerivativeReady[0][j] = true;
                }
                differentsXX[0][j] = activFuncDerivatives[0][j] * net.Layers[0].Neurons[j].Weights[0] * net.Layers[0].Neurons[j].Weights[0];
            }
            for (int i = 1; i < net.Layers.Length - 1; i++)
            {
                differentsXX[i] = new double[net.Layers[i].Neurons.Length];
                for (int j = 0; j < net.Layers[i].Neurons.Length; j++)
                {
                    double innerSecondDifferent = 0;
                    for (int k = 0; k < net.Layers[i].Neurons[j].InputsCount; k++)
                    {
                        innerSecondDifferent += net.Layers[i].Neurons[j].Weights[k] * differentsXX[i - 1][k];
                    }
                    double innerDifferent = 0;
                    for (int k = 0; k < net.Layers[i].Neurons[j].InputsCount; k++)
                    {
                        innerDifferent += net.Layers[i].Neurons[j].Weights[k] * differentsX[i - 1][k];
                    }
                    double O = 0;
                    if (!isActivFuncDerivativeReady[i][j] || !isActivFuncSecondDerivativeReady[i][j])
                    {
                        for (int k = 0; k < net.Layers[i].Neurons[j].InputsCount; k++)
                        {
                            O += net.Layers[i].Neurons[j].Weights[k] * neuronOutputs[i - 1][k];
                        }
                        O += ((ActivationNeuron)(net.Layers[i].Neurons[j])).Threshold;
                    }
                    if(!isActivFuncDerivativeReady[i][j])
                    {
                        activFuncDerivatives[i][j] = arctgFunc.Derivative(O);
                        isActivFuncDerivativeReady[i][j] = true;
                    }
                    if(!isActivFuncSecondDerivativeReady[i][j])
                    {
                        activFuncSecondDerivatives[i][j] = arctgFunc.SecondDerivative(O);
                        isActivFuncSecondDerivativeReady[i][j] = true;
                    }
                    differentsXX[i][j] = activFuncSecondDerivatives[i][j] * innerDifferent * innerDifferent + activFuncDerivatives[i][j] * innerSecondDifferent;
                }
            }
            differentsXX[net.Layers.Length - 1] = new double[net.Layers[net.Layers.Length - 1].Neurons.Length];
            for (int j = 0; j < net.Layers[net.Layers.Length - 1].Neurons.Length; j++)
            {
                double innerSecondDifferent = 0;
                for (int k = 0; k < net.Layers[net.Layers.Length - 1].Neurons[j].InputsCount; k++)
                {
                    innerSecondDifferent += net.Layers[net.Layers.Length - 1].Neurons[j].Weights[k] * differentsXX[net.Layers.Length - 2][k];
                }
                differentsXX[net.Layers.Length - 1][j] = innerSecondDifferent;
            }
            return differentsXX;
        }

        /// <summary>
        /// Вычисляет вторую производную f''xy заданного нейрона нейросети в заданной точке.
        /// </summary>
        /// <param name="net">Нейросеть.</param>
        /// <param name="outputNeuron">Заданный нейрон нейросети.</param>
        /// <param name="arctgFunc">Функция активации arctg.</param>
        /// <param name="depth">Текущая "глубина" рекурсии.</param>
        /// <param name="x">Координата X заданной точки.</param>
        /// <param name="y">Координата Y заданной точки.</param>
        /// <returns>Возвращает вычисленную вторую производную f''xy заданного нейрона нейросети в заданной точке.</returns>
        static double DifferentXY(ActivationNetwork net, ActivationNeuron outputNeuron, ArctgActivationFunction arctgFunc, int depth, double x, double y)
        {
            if (depth == net.Layers.Length)
            {
                return arctgFunc.SecondDerivative(outputNeuron.Weights[0] * x + outputNeuron.Weights[1] * y + outputNeuron.Threshold) * outputNeuron.Weights[0] * outputNeuron.Weights[1];
            }
            else
            {
                double innerSecondDifferent = 0;
                for (int i = 0; i < outputNeuron.InputsCount; i++)
                {
                    innerSecondDifferent += outputNeuron.Weights[i] * DifferentXY(net, (ActivationNeuron)net.Layers[net.Layers.Length - depth - 1].Neurons[i], arctgFunc, depth + 1, x, y);
                }
                if (depth == 1)
                {
                    return innerSecondDifferent;
                }
                else
                {
                    double innerDifferent = 0;
                    for (int i = 0; i < outputNeuron.InputsCount; i++)
                    {
                        innerDifferent += outputNeuron.Weights[i] * DifferentX(net, (ActivationNeuron)net.Layers[net.Layers.Length - depth - 1].Neurons[i], arctgFunc, depth + 1, x, y);
                    }
                    double O = 0;
                    for (int i = 0; i < outputNeuron.InputsCount; i++)
                    {
                        O += outputNeuron.Weights[i] * ComputeNeuronOutput(net, net.Layers[net.Layers.Length - depth - 1].Neurons[i], depth + 1, x, y);
                    }
                    O += outputNeuron.Threshold;
                    return arctgFunc.SecondDerivative(O) * innerDifferent * innerDifferent + arctgFunc.Derivative(O) * innerSecondDifferent;
                }
            }
        }

        /// <summary>
        /// Вычисляет вторые производные f''xy нейронов нейросети в заданной точке.
        /// </summary>
        /// <param name="net">Нейросеть.</param>
        /// <param name="arctgFunc">Функция активации arctg.</param>
        /// <param name="neuronOutputs">Выходы нейронов нейросети в заданной точке.</param>
        /// <param name="differentsX">Производные f'x выходов нейронов нейросети в заданной точке.</param>
        /// <param name="activFuncDerivatives">Производные функции активации arctg от выходов нейронов нейросети в заданной точке.</param>
        /// <param name="isActivFuncDerivativeReady">Показывает, какие производные функции активации arctg от выходов нейронов нейросети в заданной точке уже посчитаны.</param>
        /// <param name="activFuncSecondDerivatives">Вторые производные функции активации arctg от выходов нейронов нейросети в заданной точке.</param>
        /// <param name="isActivFuncSecondDerivativeReady">Показывает, какие вторые производные функции активации arctg от выходов нейронов нейросети в заданной точке уже посчитаны.</param>
        /// <param name="x">Координата X заданной точки.</param>
        /// <param name="y">Координата Y заданной точки.</param>
        /// <returns>Возвращает вычисленные вторые производные f''xy нейронов нейросети в заданной точке.</returns>
        public static double[][] DifferentsXY(ActivationNetwork net, ArctgActivationFunction arctgFunc, double[][] neuronOutputs, double[][] differentsX, ref double[][] activFuncDerivatives, ref bool[][] isActivFuncDerivativeReady, ref double[][] activFuncSecondDerivatives, ref bool[][] isActivFuncSecondDerivativeReady, double x, double y)
        {
            double[][] differentsXY = new double[net.Layers.Length][];
            differentsXY[0] = new double[net.Layers[0].Neurons.Length];
            for (int j = 0; j < net.Layers[0].Neurons.Length; j++)
            {
                if (!isActivFuncSecondDerivativeReady[0][j])
                {
                    activFuncSecondDerivatives[0][j] = arctgFunc.SecondDerivative(net.Layers[0].Neurons[j].Weights[0] * x + net.Layers[0].Neurons[j].Weights[1] * y + ((ActivationNeuron)(net.Layers[0].Neurons[j])).Threshold);
                    isActivFuncSecondDerivativeReady[0][j] = true;
                }
                differentsXY[0][j] = activFuncDerivatives[0][j] * net.Layers[0].Neurons[j].Weights[0] * net.Layers[0].Neurons[j].Weights[1];
            }
            for (int i = 1; i < net.Layers.Length - 1; i++)
            {
                differentsXY[i] = new double[net.Layers[i].Neurons.Length];
                for (int j = 0; j < net.Layers[i].Neurons.Length; j++)
                {
                    double innerSecondDifferent = 0;
                    for (int k = 0; k < net.Layers[i].Neurons[j].InputsCount; k++)
                    {
                        innerSecondDifferent += net.Layers[i].Neurons[j].Weights[k] * differentsXY[i - 1][k];
                    }
                    double innerDifferent = 0;
                    for (int k = 0; k < net.Layers[i].Neurons[j].InputsCount; k++)
                    {
                        innerDifferent += net.Layers[i].Neurons[j].Weights[k] * differentsX[i - 1][k];
                    }
                    double O = 0;
                    if (!isActivFuncDerivativeReady[i][j] || !isActivFuncSecondDerivativeReady[i][j])
                    {
                        for (int k = 0; k < net.Layers[i].Neurons[j].InputsCount; k++)
                        {
                            O += net.Layers[i].Neurons[j].Weights[k] * neuronOutputs[i - 1][k];
                        }
                        O += ((ActivationNeuron)(net.Layers[i].Neurons[j])).Threshold;
                    }
                    if (!isActivFuncDerivativeReady[i][j])
                    {
                        activFuncDerivatives[i][j] = arctgFunc.Derivative(O);
                        isActivFuncDerivativeReady[i][j] = true;
                    }
                    if (!isActivFuncSecondDerivativeReady[i][j])
                    {
                        activFuncSecondDerivatives[i][j] = arctgFunc.SecondDerivative(O);
                        isActivFuncSecondDerivativeReady[i][j] = true;
                    }
                    differentsXY[i][j] = activFuncSecondDerivatives[i][j] * innerDifferent * innerDifferent + activFuncDerivatives[i][j] * innerSecondDifferent;
                }
            }
            differentsXY[net.Layers.Length - 1] = new double[net.Layers[net.Layers.Length - 1].Neurons.Length];
            for (int j = 0; j < net.Layers[net.Layers.Length - 1].Neurons.Length; j++)
            {
                double innerSecondDifferent = 0;
                for (int k = 0; k < net.Layers[net.Layers.Length - 1].Neurons[j].InputsCount; k++)
                {
                    innerSecondDifferent += net.Layers[net.Layers.Length - 1].Neurons[j].Weights[k] * differentsXY[net.Layers.Length - 2][k];
                }
                differentsXY[net.Layers.Length - 1][j] = innerSecondDifferent;
            }
            return differentsXY;
        }

        /// <summary>
        /// Вычисляет вторую производную f''yx заданного нейрона нейросети в заданной точке.
        /// </summary>
        /// <param name="net">Нейросеть.</param>
        /// <param name="outputNeuron">Заданный нейрон нейросети.</param>
        /// <param name="arctgFunc">Функция активации arctg.</param>
        /// <param name="depth">Текущая "глубина" рекурсии.</param>
        /// <param name="x">Координата X заданной точки.</param>
        /// <param name="y">Координата Y заданной точки.</param>
        /// <returns>Возвращает вычисленную вторую производную f''yx заданного нейрона нейросети в заданной точке.</returns>
        static double DifferentYX(ActivationNetwork net, ActivationNeuron outputNeuron, ArctgActivationFunction arctgFunc, int depth, double x, double y)
        {
            if (depth == net.Layers.Length)
            {
                return arctgFunc.SecondDerivative(outputNeuron.Weights[0] * x + outputNeuron.Weights[1] * y + outputNeuron.Threshold) * outputNeuron.Weights[0] * outputNeuron.Weights[1];
            }
            else
            {
                double innerSecondDifferent = 0;
                for (int i = 0; i < outputNeuron.InputsCount; i++)
                {
                    innerSecondDifferent += outputNeuron.Weights[i] * DifferentYX(net, (ActivationNeuron)net.Layers[net.Layers.Length - depth - 1].Neurons[i], arctgFunc, depth + 1, x, y);
                }
                if (depth == 1)
                {
                    return innerSecondDifferent;
                }
                else
                {
                    double innerDifferent = 0;
                    for (int i = 0; i < outputNeuron.InputsCount; i++)
                    {
                        innerDifferent += outputNeuron.Weights[i] * DifferentY(net, (ActivationNeuron)net.Layers[net.Layers.Length - depth - 1].Neurons[i], arctgFunc, depth + 1, x, y);
                    }
                    double O = 0;
                    for (int i = 0; i < outputNeuron.InputsCount; i++)
                    {
                        O += outputNeuron.Weights[i] * ComputeNeuronOutput(net, net.Layers[net.Layers.Length - depth - 1].Neurons[i], depth + 1, x, y);
                    }
                    O += outputNeuron.Threshold;
                    return arctgFunc.SecondDerivative(O) * innerDifferent * innerDifferent + arctgFunc.Derivative(O) * innerSecondDifferent;
                }
            }
        }

        /// <summary>
        /// Вычисляет вторые производные f''yx нейронов нейросети в заданной точке.
        /// </summary>
        /// <param name="net">Нейросеть.</param>
        /// <param name="arctgFunc">Функция активации arctg.</param>
        /// <param name="neuronOutputs">Выходы нейронов нейросети в заданной точке.</param>
        /// <param name="differentsY">Производные f'y выходов нейронов нейросети в заданной точке.</param>
        /// <param name="activFuncDerivatives">Производные функции активации arctg от выходов нейронов нейросети в заданной точке.</param>
        /// <param name="isActivFuncDerivativeReady">Показывает, какие производные функции активации arctg от выходов нейронов нейросети в заданной точке уже посчитаны.</param>
        /// <param name="activFuncSecondDerivatives">Вторые производные функции активации arctg от выходов нейронов нейросети в заданной точке.</param>
        /// <param name="isActivFuncSecondDerivativeReady">Показывает, какие вторые производные функции активации arctg от выходов нейронов нейросети в заданной точке уже посчитаны.</param>
        /// <param name="x">Координата X заданной точки.</param>
        /// <param name="y">Координата Y заданной точки.</param>
        /// <returns>Возвращает вычисленные вторые производные f''yx нейронов нейросети в заданной точке.</returns>
        public static double[][] DifferentsYX(ActivationNetwork net, ArctgActivationFunction arctgFunc, double[][] neuronOutputs, double[][] differentsY, ref double[][] activFuncDerivatives, ref bool[][] isActivFuncDerivativeReady, ref double[][] activFuncSecondDerivatives, ref bool[][] isActivFuncSecondDerivativeReady, double x, double y)
        {
            double[][] differentsYX = new double[net.Layers.Length][];
            differentsYX[0] = new double[net.Layers[0].Neurons.Length];
            for (int j = 0; j < net.Layers[0].Neurons.Length; j++)
            {
                if (!isActivFuncSecondDerivativeReady[0][j])
                {
                    activFuncSecondDerivatives[0][j] = arctgFunc.SecondDerivative(net.Layers[0].Neurons[j].Weights[0] * x + net.Layers[0].Neurons[j].Weights[1] * y + ((ActivationNeuron)(net.Layers[0].Neurons[j])).Threshold);
                    isActivFuncSecondDerivativeReady[0][j] = true;
                }
                differentsYX[0][j] = activFuncDerivatives[0][j] * net.Layers[0].Neurons[j].Weights[1] * net.Layers[0].Neurons[j].Weights[0];
            }
            for (int i = 1; i < net.Layers.Length - 1; i++)
            {
                differentsYX[i] = new double[net.Layers[i].Neurons.Length];
                for (int j = 0; j < net.Layers[i].Neurons.Length; j++)
                {
                    double innerSecondDifferent = 0;
                    for (int k = 0; k < net.Layers[i].Neurons[j].InputsCount; k++)
                    {
                        innerSecondDifferent += net.Layers[i].Neurons[j].Weights[k] * differentsYX[i - 1][k];
                    }
                    double innerDifferent = 0;
                    for (int k = 0; k < net.Layers[i].Neurons[j].InputsCount; k++)
                    {
                        innerDifferent += net.Layers[i].Neurons[j].Weights[k] * differentsY[i - 1][k];
                    }
                    double O = 0;
                    if (!isActivFuncDerivativeReady[i][j] || !isActivFuncSecondDerivativeReady[i][j])
                    {
                        for (int k = 0; k < net.Layers[i].Neurons[j].InputsCount; k++)
                        {
                            O += net.Layers[i].Neurons[j].Weights[k] * neuronOutputs[i - 1][k];
                        }
                        O += ((ActivationNeuron)(net.Layers[i].Neurons[j])).Threshold;
                    }
                    if (!isActivFuncDerivativeReady[i][j])
                    {
                        activFuncDerivatives[i][j] = arctgFunc.Derivative(O);
                        isActivFuncDerivativeReady[i][j] = true;
                    }
                    if (!isActivFuncSecondDerivativeReady[i][j])
                    {
                        activFuncSecondDerivatives[i][j] = arctgFunc.SecondDerivative(O);
                        isActivFuncSecondDerivativeReady[i][j] = true;
                    }
                    differentsYX[i][j] = activFuncSecondDerivatives[i][j] * innerDifferent * innerDifferent + activFuncDerivatives[i][j] * innerSecondDifferent;
                }
            }
            differentsYX[net.Layers.Length - 1] = new double[net.Layers[net.Layers.Length - 1].Neurons.Length];
            for (int j = 0; j < net.Layers[net.Layers.Length - 1].Neurons.Length; j++)
            {
                double innerSecondDifferent = 0;
                for (int k = 0; k < net.Layers[net.Layers.Length - 1].Neurons[j].InputsCount; k++)
                {
                    innerSecondDifferent += net.Layers[net.Layers.Length - 1].Neurons[j].Weights[k] * differentsYX[net.Layers.Length - 2][k];
                }
                differentsYX[net.Layers.Length - 1][j] = innerSecondDifferent;
            }
            return differentsYX;
        }

        /// <summary>
        /// Вычисляет вторую производную f''yy заданного нейрона нейросети в заданной точке.
        /// </summary>
        /// <param name="net">Нейросеть.</param>
        /// <param name="outputNeuron">Заданный нейрон нейросети.</param>
        /// <param name="arctgFunc">Функция активации arctg.</param>
        /// <param name="depth">Текущая "глубина" рекурсии.</param>
        /// <param name="x">Координата X заданной точки.</param>
        /// <param name="y">Координата Y заданной точки.</param>
        /// <returns>Возвращает вычисленную вторую производную f''yy заданного нейрона нейросети в заданной точке.</returns>
        static double DifferentYY(ActivationNetwork net, ActivationNeuron outputNeuron, ArctgActivationFunction arctgFunc, int depth, double x, double y)
        {
            if (depth == net.Layers.Length)
            {
                return arctgFunc.SecondDerivative(outputNeuron.Weights[0] * x + outputNeuron.Weights[1] * y + outputNeuron.Threshold) * outputNeuron.Weights[1] * outputNeuron.Weights[1];
            }
            else
            {
                double innerSecondDifferent = 0;
                for (int i = 0; i < outputNeuron.InputsCount; i++)
                {
                    innerSecondDifferent += outputNeuron.Weights[i] * DifferentYY(net, (ActivationNeuron)net.Layers[net.Layers.Length - depth - 1].Neurons[i], arctgFunc, depth + 1, x, y);
                }
                if (depth == 1)
                {
                    return innerSecondDifferent;
                }
                else
                {
                    double innerDifferent = 0;
                    for (int i = 0; i < outputNeuron.InputsCount; i++)
                    {
                        innerDifferent += outputNeuron.Weights[i] * DifferentY(net, (ActivationNeuron)net.Layers[net.Layers.Length - depth - 1].Neurons[i], arctgFunc, depth + 1, x, y);
                    }
                    double O = 0;
                    for (int i = 0; i < outputNeuron.InputsCount; i++)
                    {
                        O += outputNeuron.Weights[i] * ComputeNeuronOutput(net, net.Layers[net.Layers.Length - depth - 1].Neurons[i], depth + 1, x, y);
                    }
                    O += outputNeuron.Threshold;
                    return arctgFunc.SecondDerivative(O) * innerDifferent * innerDifferent + arctgFunc.Derivative(O) * innerSecondDifferent;
                }
            }
        }

        /// <summary>
        /// Вычисляет вторые производные f''yy нейронов нейросети в заданной точке.
        /// </summary>
        /// <param name="net">Нейросеть.</param>
        /// <param name="arctgFunc">Функция активации arctg.</param>
        /// <param name="neuronOutputs">Выходы нейронов нейросети в заданной точке.</param>
        /// <param name="differentsY">Производные f'y выходов нейронов нейросети в заданной точке.</param>
        /// <param name="activFuncDerivatives">Производные функции активации arctg от выходов нейронов нейросети в заданной точке.</param>
        /// <param name="isActivFuncDerivativeReady">Показывает, какие производные функции активации arctg от выходов нейронов нейросети в заданной точке уже посчитаны.</param>
        /// <param name="activFuncSecondDerivatives">Вторые производные функции активации arctg от выходов нейронов нейросети в заданной точке.</param>
        /// <param name="isActivFuncSecondDerivativeReady">Показывает, какие вторые производные функции активации arctg от выходов нейронов нейросети в заданной точке уже посчитаны.</param>
        /// <param name="x">Координата X заданной точки.</param>
        /// <param name="y">Координата Y заданной точки.</param>
        /// <returns>Возвращает вычисленные вторые производные f''yy нейронов нейросети в заданной точке.</returns>
        public static double[][] DifferentsYY(ActivationNetwork net, ArctgActivationFunction arctgFunc, double[][] neuronOutputs, double[][] differentsY, ref double[][] activFuncDerivatives, ref bool[][] isActivFuncDerivativeReady, ref double[][] activFuncSecondDerivatives, ref bool[][] isActivFuncSecondDerivativeReady, double x, double y)
        {
            double[][] differentsYY = new double[net.Layers.Length][];
            differentsYY[0] = new double[net.Layers[0].Neurons.Length];
            for (int j = 0; j < net.Layers[0].Neurons.Length; j++)
            {
                if (!isActivFuncSecondDerivativeReady[0][j])
                {
                    activFuncSecondDerivatives[0][j] = arctgFunc.SecondDerivative(net.Layers[0].Neurons[j].Weights[0] * x + net.Layers[0].Neurons[j].Weights[1] * y + ((ActivationNeuron)(net.Layers[0].Neurons[j])).Threshold);
                    isActivFuncSecondDerivativeReady[0][j] = true;
                }
                differentsYY[0][j] = activFuncDerivatives[0][j] * net.Layers[0].Neurons[j].Weights[1] * net.Layers[0].Neurons[j].Weights[0];
            }
            for (int i = 1; i < net.Layers.Length - 1; i++)
            {
                differentsYY[i] = new double[net.Layers[i].Neurons.Length];
                for (int j = 0; j < net.Layers[i].Neurons.Length; j++)
                {
                    double innerSecondDifferent = 0;
                    for (int k = 0; k < net.Layers[i].Neurons[j].InputsCount; k++)
                    {
                        innerSecondDifferent += net.Layers[i].Neurons[j].Weights[k] * differentsYY[i - 1][k];
                    }
                    double innerDifferent = 0;
                    for (int k = 0; k < net.Layers[i].Neurons[j].InputsCount; k++)
                    {
                        innerDifferent += net.Layers[i].Neurons[j].Weights[k] * differentsY[i - 1][k];
                    }
                    double O = 0;
                    if (!isActivFuncDerivativeReady[i][j] || !isActivFuncSecondDerivativeReady[i][j])
                    {
                        for (int k = 0; k < net.Layers[i].Neurons[j].InputsCount; k++)
                        {
                            O += net.Layers[i].Neurons[j].Weights[k] * neuronOutputs[i - 1][k];
                        }
                        O += ((ActivationNeuron)(net.Layers[i].Neurons[j])).Threshold;
                    }
                    if (!isActivFuncDerivativeReady[i][j])
                    {
                        activFuncDerivatives[i][j] = arctgFunc.Derivative(O);
                        isActivFuncDerivativeReady[i][j] = true;
                    }
                    if (!isActivFuncSecondDerivativeReady[i][j])
                    {
                        activFuncSecondDerivatives[i][j] = arctgFunc.SecondDerivative(O);
                        isActivFuncSecondDerivativeReady[i][j] = true;
                    }
                    differentsYY[i][j] = activFuncSecondDerivatives[i][j] * innerDifferent * innerDifferent + activFuncDerivatives[i][j] * innerSecondDifferent;
                }
            }
            differentsYY[net.Layers.Length - 1] = new double[net.Layers[net.Layers.Length - 1].Neurons.Length];
            for (int j = 0; j < net.Layers[net.Layers.Length - 1].Neurons.Length; j++)
            {
                double innerSecondDifferent = 0;
                for (int k = 0; k < net.Layers[net.Layers.Length - 1].Neurons[j].InputsCount; k++)
                {
                    innerSecondDifferent += net.Layers[net.Layers.Length - 1].Neurons[j].Weights[k] * differentsYY[net.Layers.Length - 2][k];
                }
                differentsYY[net.Layers.Length - 1][j] = innerSecondDifferent;
            }
            return differentsYY;
        }

        /// <summary>
        /// Вычисляет производную заданного нейрона нейросети по заданному весовому коэффициенту в заданной точке.
        /// </summary>
        /// <param name="net">Нейросеть.</param>
        /// <param name="outputNeuron">Заданный нейрон нейросети.</param>
        /// <param name="arctgFunc">Функция активации arctg.</param>
        /// <param name="neuronOutputs">Выходы нейронов нейросети в заданной точке.</param>
        /// <param name="currentLayer">Номер слоя заданного нейрона.</param>
        /// <param name="currentNeuronInCurrentLayer">Номер заданного нейрона в слое.</param>
        /// <param name="activFuncDerivatives">Производные функции активации arctg от выходов нейронов нейросети в заданной точке.</param>
        /// <param name="isActivFuncDerivativeReady">Показывает, какие производные функции активации arctg от выходов нейронов нейросети в заданной точке уже посчитаны.</param>
        /// <param name="layerNumber">Номер слоя заданного весового коэффициента.</param>
        /// <param name="neuronInLayerNumber">Номер нейрона заданного весового коэффициента в слое.</param>
        /// <param name="weightInNeuronNumber">Номер заданного весового коэффициента в нейроне.</param>
        /// <param name="x">Координата X заданной точки.</param>
        /// <param name="y">Координата Y заданной точки.</param>
        /// <returns>Возвращает вычисленную производную заданного нейрона нейросети по заданному весовому коэффициенту в заданной точке.</returns>
        static double DifferentWeight(ActivationNetwork net, ActivationNeuron outputNeuron, ArctgActivationFunction arctgFunc, double[][] neuronOutputs, int currentLayer, int currentNeuronInCurrentLayer, ref double[][] activFuncDerivatives, ref bool[][] isActivFuncDerivativeReady, int layerNumber, int neuronInLayerNumber, int weightInNeuronNumber, double x, double y)
        {
            if(currentLayer == 0)
            {
                if (!isActivFuncDerivativeReady[currentLayer][currentNeuronInCurrentLayer])
                {
                    activFuncDerivatives[currentLayer][currentNeuronInCurrentLayer] = arctgFunc.Derivative(outputNeuron.Weights[0] * x + outputNeuron.Weights[1] * y + outputNeuron.Threshold);
                    isActivFuncDerivativeReady[currentLayer][currentNeuronInCurrentLayer] = true;
                }
                if (weightInNeuronNumber == 0)
                {
                    return activFuncDerivatives[currentLayer][currentNeuronInCurrentLayer] * x;
                }
                else if (weightInNeuronNumber == 1)
                {
                    return activFuncDerivatives[currentLayer][currentNeuronInCurrentLayer] * y;
                }
                else
                {
                    return activFuncDerivatives[currentLayer][currentNeuronInCurrentLayer];
                }
            }
            else
            {
                if(currentLayer == layerNumber)
                {
                    if (currentLayer == net.Layers.Length - 1)
                    {
                        if (weightInNeuronNumber == outputNeuron.Weights.Length)
                        {
                            return 1;
                        }
                        else
                        {
                            return neuronOutputs[currentLayer - 1][weightInNeuronNumber];
                        }
                    }
                    else
                    {
                        if (!isActivFuncDerivativeReady[currentLayer][currentNeuronInCurrentLayer])
                        {
                            double O = 0;
                            for (int i = 0; i < outputNeuron.InputsCount; i++)
                            {
                                O += outputNeuron.Weights[i] * neuronOutputs[currentLayer - 1][i];
                            }
                            O += outputNeuron.Threshold;
                            activFuncDerivatives[currentLayer][currentNeuronInCurrentLayer] = arctgFunc.Derivative(O);
                            isActivFuncDerivativeReady[currentLayer][currentNeuronInCurrentLayer] = true;
                        }
                        if (weightInNeuronNumber == outputNeuron.Weights.Length)
                        {
                            return activFuncDerivatives[currentLayer][currentNeuronInCurrentLayer];
                        }
                        else
                        {
                            return activFuncDerivatives[currentLayer][currentNeuronInCurrentLayer] * neuronOutputs[currentLayer - 1][weightInNeuronNumber];
                        }
                    }
                }
                else
                {
                    if (currentLayer == net.Layers.Length - 1)
                    {
                        if(currentLayer - layerNumber == 1)
                        {
                            return outputNeuron.Weights[neuronInLayerNumber] * DifferentWeight(net, (ActivationNeuron)net.Layers[currentLayer - 1].Neurons[neuronInLayerNumber], arctgFunc, neuronOutputs, currentLayer - 1, neuronInLayerNumber, ref activFuncDerivatives, ref isActivFuncDerivativeReady, layerNumber, neuronInLayerNumber, weightInNeuronNumber, x, y);
                        }
                        else
                        {
                            double innerDifferent = 0;
                            for(int i = 0; i < outputNeuron.InputsCount; i++)
                            {
                                innerDifferent += outputNeuron.Weights[i] * DifferentWeight(net, (ActivationNeuron)net.Layers[currentLayer - 1].Neurons[i], arctgFunc, neuronOutputs, currentLayer - 1, i, ref activFuncDerivatives, ref isActivFuncDerivativeReady, layerNumber, neuronInLayerNumber, weightInNeuronNumber, x, y);
                            }
                            return innerDifferent;
                        }
                    }
                    else
                    {
                        if (!isActivFuncDerivativeReady[currentLayer][currentNeuronInCurrentLayer])
                        {
                            double O = 0;
                            for (int i = 0; i < outputNeuron.InputsCount; i++)
                            {
                                O += outputNeuron.Weights[i] * neuronOutputs[currentLayer - 1][i];
                            }
                            O += outputNeuron.Threshold;
                            activFuncDerivatives[currentLayer][currentNeuronInCurrentLayer] = arctgFunc.Derivative(O);
                            isActivFuncDerivativeReady[currentLayer][currentNeuronInCurrentLayer] = true;
                        }
                        if (currentLayer - layerNumber == 1)
                        {
                            return activFuncDerivatives[currentLayer][currentNeuronInCurrentLayer] * outputNeuron.Weights[neuronInLayerNumber] * DifferentWeight(net, (ActivationNeuron)net.Layers[currentLayer - 1].Neurons[neuronInLayerNumber], arctgFunc, neuronOutputs, currentLayer - 1, neuronInLayerNumber, ref activFuncDerivatives, ref isActivFuncDerivativeReady, layerNumber, neuronInLayerNumber, weightInNeuronNumber, x, y);
                        }
                        else
                        {
                            double innerWeightDifferent = 0;
                            for (int i = 0; i < outputNeuron.InputsCount; i++)
                            {
                                innerWeightDifferent += outputNeuron.Weights[i] * DifferentWeight(net, (ActivationNeuron)net.Layers[currentLayer - 1].Neurons[i], arctgFunc, neuronOutputs, currentLayer - 1, i, ref activFuncDerivatives, ref isActivFuncDerivativeReady, layerNumber, neuronInLayerNumber, weightInNeuronNumber, x, y);
                            }
                            return activFuncDerivatives[currentLayer][currentNeuronInCurrentLayer] * innerWeightDifferent;
                        }
                    }
                }
            }
        }

        /// <summary>
        /// Вычисляет производную производной f'x нейросети по заданному весовому коэффициенту в заданной точке.
        /// </summary>
        /// <param name="net">Нейросеть.</param>
        /// <param name="outputNeuron">Заданный нейрон нейросети.</param>
        /// <param name="arctgFunc">Функция активации arctg.</param>
        /// <param name="neuronOutputs">Выходы нейронов нейросети в заданной точке.</param>
        /// <param name="differentsX">Производные f'x выходов нейронов нейросети в заданной точке.</param>
        /// <param name="currentLayer">Номер слоя заданного нейрона.</param>
        /// <param name="currentNeuronInCurrentLayer">Номер заданного нейрона в слое.</param>
        /// <param name="activFuncDerivatives">Производные функции активации arctg от выходов нейронов нейросети в заданной точке.</param>
        /// <param name="isActivFuncDerivativeReady">Показывает, какие производные функции активации arctg от выходов нейронов нейросети в заданной точке уже посчитаны.</param>
        /// <param name="activFuncSecondDerivatives">Вторые производные функции активации arctg от выходов нейронов нейросети в заданной точке.</param>
        /// <param name="isActivFuncSecondDerivativeReady">Показывает, какие вторые производные функции активации arctg от выходов нейронов нейросети в заданной точке уже посчитаны.</param>
        /// <param name="layerNumber">Номер слоя заданного весового коэффициента.</param>
        /// <param name="neuronInLayerNumber">Номер нейрона заданного весового коэффициента в слое.</param>
        /// <param name="weightInNeuronNumber">Номер заданного весового коэффициента в нейроне.</param>
        /// <param name="x">Координата X заданной точки.</param>
        /// <param name="y">Координата Y заданной точки.</param>
        /// <returns>Возвращает вычисленную производную производной f'x нейросети по заданному весовому коэффициенту в заданной точке.</returns>
        static double DifferentXWeight(ActivationNetwork net, ActivationNeuron outputNeuron, ArctgActivationFunction arctgFunc, double[][] neuronOutputs, double[][] differentsX, int currentLayer, int currentNeuronInCurrentLayer, ref double[][] activFuncDerivatives, ref bool[][] isActivFuncDerivativeReady, ref double[][] activFuncSecondDerivatives, ref bool[][] isActivFuncSecondDerivativeReady, int layerNumber, int neuronInLayerNumber, int weightInNeuronNumber, double x, double y)
        {
            if(currentLayer == 0)
            {
                if(!isActivFuncSecondDerivativeReady[currentLayer][currentNeuronInCurrentLayer])
                {
                    activFuncSecondDerivatives[currentLayer][currentNeuronInCurrentLayer] = arctgFunc.SecondDerivative(outputNeuron.Weights[0] * x + outputNeuron.Weights[1] * y + outputNeuron.Threshold);
                    isActivFuncSecondDerivativeReady[currentLayer][currentNeuronInCurrentLayer] = true;
                }
                if (weightInNeuronNumber == 0)
                {
                    return activFuncSecondDerivatives[currentLayer][currentNeuronInCurrentLayer] * outputNeuron.Weights[0] * x;
                }
                else if (weightInNeuronNumber == 1)
                {
                    return activFuncSecondDerivatives[currentLayer][currentNeuronInCurrentLayer] * outputNeuron.Weights[0] * y;
                }
                else
                {
                    return activFuncSecondDerivatives[currentLayer][currentNeuronInCurrentLayer] * outputNeuron.Weights[0];
                }
            }
            else
            {
                if(currentLayer == layerNumber)
                {
                    if (currentLayer == net.Layers.Length - 1)
                    {
                        if (weightInNeuronNumber == outputNeuron.Weights.Length)
                        {
                            return 0;
                        }
                        else
                        {
                            return differentsX[currentLayer - 1][weightInNeuronNumber];
                        }
                    }
                    else
                    {
                        double O = 0;
                        if (!isActivFuncDerivativeReady[currentLayer][currentNeuronInCurrentLayer] || !isActivFuncSecondDerivativeReady[currentLayer][currentNeuronInCurrentLayer])
                        {
                            for (int i = 0; i < outputNeuron.InputsCount; i++)
                            {
                                O += outputNeuron.Weights[i] * neuronOutputs[currentLayer - 1][i];
                            }
                            O += outputNeuron.Threshold;
                        }
                        if(!isActivFuncDerivativeReady[currentLayer][currentNeuronInCurrentLayer])
                        {
                            activFuncDerivatives[currentLayer][currentNeuronInCurrentLayer] = arctgFunc.Derivative(O);
                            isActivFuncDerivativeReady[currentLayer][currentNeuronInCurrentLayer] = true;
                        }
                        if(!isActivFuncSecondDerivativeReady[currentLayer][currentNeuronInCurrentLayer])
                        {
                            activFuncSecondDerivatives[currentLayer][currentNeuronInCurrentLayer] = arctgFunc.SecondDerivative(O);
                            isActivFuncSecondDerivativeReady[currentLayer][currentNeuronInCurrentLayer] = true;
                        }
                        double innerDifferent = 0;
                        for (int i = 0; i < outputNeuron.InputsCount; i++)
                        {
                            innerDifferent += outputNeuron.Weights[i] * differentsX[currentLayer - 1][i];
                        }
                        if (weightInNeuronNumber == outputNeuron.Weights.Length)
                        {
                            return activFuncSecondDerivatives[currentLayer][currentNeuronInCurrentLayer] * innerDifferent;
                        }
                        else
                        {
                            return activFuncSecondDerivatives[currentLayer][currentNeuronInCurrentLayer] * neuronOutputs[currentLayer - 1][weightInNeuronNumber] * innerDifferent + activFuncDerivatives[currentLayer][currentNeuronInCurrentLayer] * differentsX[currentLayer - 1][weightInNeuronNumber];
                        }
                    }
                }
                else
                {
                    if (currentLayer == net.Layers.Length - 1)
                    {
                        if (currentLayer - layerNumber == 1)
                        {
                            return outputNeuron.Weights[neuronInLayerNumber] * DifferentXWeight(net, (ActivationNeuron)net.Layers[currentLayer - 1].Neurons[neuronInLayerNumber], arctgFunc, neuronOutputs, differentsX, currentLayer - 1, neuronInLayerNumber, ref activFuncDerivatives, ref isActivFuncDerivativeReady, ref activFuncSecondDerivatives, ref isActivFuncSecondDerivativeReady, layerNumber, neuronInLayerNumber, weightInNeuronNumber, x, y);
                        }
                        else
                        {
                            double innerWeightDifferent = 0;
                            for (int i = 0; i < outputNeuron.InputsCount; i++)
                            {
                                innerWeightDifferent += outputNeuron.Weights[i] * DifferentXWeight(net, (ActivationNeuron)net.Layers[currentLayer - 1].Neurons[i], arctgFunc, neuronOutputs, differentsX, currentLayer - 1, i, ref activFuncDerivatives, ref isActivFuncDerivativeReady, ref activFuncSecondDerivatives, ref isActivFuncSecondDerivativeReady, layerNumber, neuronInLayerNumber, weightInNeuronNumber, x, y);
                            }
                            return innerWeightDifferent;
                        }
                    }
                    else
                    {
                        double O = 0;
                        if (!isActivFuncDerivativeReady[currentLayer][currentNeuronInCurrentLayer] || !isActivFuncSecondDerivativeReady[currentLayer][currentNeuronInCurrentLayer])
                        {
                            for (int i = 0; i < outputNeuron.InputsCount; i++)
                            {
                                O += outputNeuron.Weights[i] * neuronOutputs[currentLayer - 1][i];
                            }
                            O += outputNeuron.Threshold;
                        }
                        if(!isActivFuncDerivativeReady[currentLayer][currentNeuronInCurrentLayer])
                        {
                            activFuncDerivatives[currentLayer][currentNeuronInCurrentLayer] = arctgFunc.Derivative(O);
                            isActivFuncDerivativeReady[currentLayer][currentNeuronInCurrentLayer] = true;
                        }
                        if(!isActivFuncSecondDerivativeReady[currentLayer][currentNeuronInCurrentLayer])
                        {
                            activFuncSecondDerivatives[currentLayer][currentNeuronInCurrentLayer] = arctgFunc.SecondDerivative(O);
                            isActivFuncSecondDerivativeReady[currentLayer][currentNeuronInCurrentLayer] = true;
                        }
                        double innerDifferent = 0;
                        for (int i = 0; i < outputNeuron.InputsCount; i++)
                        {
                            innerDifferent += outputNeuron.Weights[i] * differentsX[currentLayer - 1][i];
                        }
                        double leftPartInnerWeightDifferent = 0;
                        double rightPartInnerWeightDifferent = 0;
                        if (currentLayer - layerNumber == 1)
                        {
                            leftPartInnerWeightDifferent = outputNeuron.Weights[neuronInLayerNumber] * DifferentWeight(net, (ActivationNeuron)net.Layers[currentLayer - 1].Neurons[neuronInLayerNumber], arctgFunc, neuronOutputs, currentLayer - 1, neuronInLayerNumber, ref activFuncDerivatives, ref isActivFuncDerivativeReady, layerNumber, neuronInLayerNumber, weightInNeuronNumber, x, y);
                            rightPartInnerWeightDifferent = outputNeuron.Weights[neuronInLayerNumber] * DifferentXWeight(net, (ActivationNeuron)net.Layers[currentLayer - 1].Neurons[neuronInLayerNumber], arctgFunc, neuronOutputs, differentsX, currentLayer - 1, neuronInLayerNumber, ref activFuncDerivatives, ref isActivFuncDerivativeReady, ref activFuncSecondDerivatives, ref isActivFuncSecondDerivativeReady, layerNumber, neuronInLayerNumber, weightInNeuronNumber, x, y);
                        }
                        else
                        {
                            for (int i = 0; i < outputNeuron.InputsCount; i++)
                            {
                                leftPartInnerWeightDifferent += outputNeuron.Weights[i] * DifferentWeight(net, (ActivationNeuron)net.Layers[currentLayer - 1].Neurons[i], arctgFunc, neuronOutputs, currentLayer - 1, i, ref activFuncDerivatives, ref isActivFuncDerivativeReady, layerNumber, neuronInLayerNumber, weightInNeuronNumber, x, y);
                                rightPartInnerWeightDifferent += outputNeuron.Weights[i] * DifferentXWeight(net, (ActivationNeuron)net.Layers[currentLayer - 1].Neurons[i], arctgFunc, neuronOutputs, differentsX, currentLayer - 1, i, ref activFuncDerivatives, ref isActivFuncDerivativeReady, ref activFuncSecondDerivatives, ref isActivFuncSecondDerivativeReady, layerNumber, neuronInLayerNumber, weightInNeuronNumber, x, y);
                            }
                        }
                        return activFuncSecondDerivatives[currentLayer][currentNeuronInCurrentLayer] * leftPartInnerWeightDifferent * innerDifferent + activFuncDerivatives[currentLayer][currentNeuronInCurrentLayer] * rightPartInnerWeightDifferent;
                    }
                }
            }
        }

        /// <summary>
        /// Вычисляет производную производной f'y нейросети по заданному весовому коэффициенту в заданной точке.
        /// </summary>
        /// <param name="net">Нейросеть.</param>
        /// <param name="outputNeuron">Заданный нейрон нейросети.</param>
        /// <param name="arctgFunc">Функция активации arctg.</param>
        /// <param name="neuronOutputs">Выходы нейронов нейросети в заданной точке.</param>
        /// <param name="differentsY">Производные f'y выходов нейронов нейросети в заданной точке.</param>
        /// <param name="currentLayer">Номер слоя заданного нейрона.</param>
        /// <param name="currentNeuronInCurrentLayer">Номер заданного нейрона в слое.</param>
        /// <param name="activFuncDerivatives">Производные функции активации arctg от выходов нейронов нейросети в заданной точке.</param>
        /// <param name="isActivFuncDerivativeReady">Показывает, какие производные функции активации arctg от выходов нейронов нейросети в заданной точке уже посчитаны.</param>
        /// <param name="activFuncSecondDerivatives">Вторые производные функции активации arctg от выходов нейронов нейросети в заданной точке.</param>
        /// <param name="isActivFuncSecondDerivativeReady">Показывает, какие вторые производные функции активации arctg от выходов нейронов нейросети в заданной точке уже посчитаны.</param>
        /// <param name="layerNumber">Номер слоя заданного весового коэффициента.</param>
        /// <param name="neuronInLayerNumber">Номер нейрона заданного весового коэффициента в слое.</param>
        /// <param name="weightInNeuronNumber">Номер заданного весового коэффициента в нейроне.</param>
        /// <param name="x">Координата X заданной точки.</param>
        /// <param name="y">Координата Y заданной точки.</param>
        /// <returns>Возвращает вычисленную производную производной f'y нейросети по заданному весовому коэффициенту в заданной точке.</returns>
        static double DifferentYWeight(ActivationNetwork net, ActivationNeuron outputNeuron, ArctgActivationFunction arctgFunc, double[][] neuronOutputs, double[][] differentsY, int currentLayer, int currentNeuronInCurrentLayer, ref double[][] activFuncDerivatives, ref bool[][] isActivFuncDerivativeReady, ref double[][] activFuncSecondDerivatives, ref bool[][] isActivFuncSecondDerivativeReady, int layerNumber, int neuronInLayerNumber, int weightInNeuronNumber, double x, double y)
        {
            if (currentLayer == 0)
            {
                if (!isActivFuncSecondDerivativeReady[currentLayer][currentNeuronInCurrentLayer])
                {
                    activFuncSecondDerivatives[currentLayer][currentNeuronInCurrentLayer] = arctgFunc.SecondDerivative(outputNeuron.Weights[0] * x + outputNeuron.Weights[1] * y + outputNeuron.Threshold);
                    isActivFuncSecondDerivativeReady[currentLayer][currentNeuronInCurrentLayer] = true;
                }
                if (weightInNeuronNumber == 0)
                {
                    return activFuncSecondDerivatives[currentLayer][currentNeuronInCurrentLayer] * outputNeuron.Weights[0] * x;
                }
                else if (weightInNeuronNumber == 1)
                {
                    return activFuncSecondDerivatives[currentLayer][currentNeuronInCurrentLayer] * outputNeuron.Weights[0] * y;
                }
                else
                {
                    return activFuncSecondDerivatives[currentLayer][currentNeuronInCurrentLayer] * outputNeuron.Weights[0];
                }
            }
            else
            {
                if (currentLayer == layerNumber)
                {
                    if (currentLayer == net.Layers.Length - 1)
                    {
                        if (weightInNeuronNumber == outputNeuron.Weights.Length)
                        {
                            return 0;
                        }
                        else
                        {
                            return differentsY[currentLayer - 1][weightInNeuronNumber];
                        }
                    }
                    else
                    {
                        double O = 0;
                        if (!isActivFuncDerivativeReady[currentLayer][currentNeuronInCurrentLayer] || !isActivFuncSecondDerivativeReady[currentLayer][currentNeuronInCurrentLayer])
                        {
                            for (int i = 0; i < outputNeuron.InputsCount; i++)
                            {
                                O += outputNeuron.Weights[i] * neuronOutputs[currentLayer - 1][i];
                            }
                            O += outputNeuron.Threshold;
                        }
                        if (!isActivFuncDerivativeReady[currentLayer][currentNeuronInCurrentLayer])
                        {
                            activFuncDerivatives[currentLayer][currentNeuronInCurrentLayer] = arctgFunc.Derivative(O);
                            isActivFuncDerivativeReady[currentLayer][currentNeuronInCurrentLayer] = true;
                        }
                        if (!isActivFuncSecondDerivativeReady[currentLayer][currentNeuronInCurrentLayer])
                        {
                            activFuncSecondDerivatives[currentLayer][currentNeuronInCurrentLayer] = arctgFunc.SecondDerivative(O);
                            isActivFuncSecondDerivativeReady[currentLayer][currentNeuronInCurrentLayer] = true;
                        }
                        double innerDifferent = 0;
                        for (int i = 0; i < outputNeuron.InputsCount; i++)
                        {
                            innerDifferent += outputNeuron.Weights[i] * differentsY[currentLayer - 1][i];
                        }
                        if (weightInNeuronNumber == outputNeuron.Weights.Length)
                        {
                            return activFuncSecondDerivatives[currentLayer][currentNeuronInCurrentLayer] * innerDifferent;
                        }
                        else
                        {
                            return activFuncSecondDerivatives[currentLayer][currentNeuronInCurrentLayer] * neuronOutputs[currentLayer - 1][weightInNeuronNumber] * innerDifferent + activFuncDerivatives[currentLayer][currentNeuronInCurrentLayer] * differentsY[currentLayer - 1][weightInNeuronNumber];
                        }
                    }
                }
                else
                {
                    if (currentLayer == net.Layers.Length - 1)
                    {
                        if (currentLayer - layerNumber == 1)
                        {
                            return outputNeuron.Weights[neuronInLayerNumber] * DifferentYWeight(net, (ActivationNeuron)net.Layers[currentLayer - 1].Neurons[neuronInLayerNumber], arctgFunc, neuronOutputs, differentsY, currentLayer - 1, neuronInLayerNumber, ref activFuncDerivatives, ref isActivFuncDerivativeReady, ref activFuncSecondDerivatives, ref isActivFuncSecondDerivativeReady, layerNumber, neuronInLayerNumber, weightInNeuronNumber, x, y);
                        }
                        else
                        {
                            double innerWeightDifferent = 0;
                            for (int i = 0; i < outputNeuron.InputsCount; i++)
                            {
                                innerWeightDifferent += outputNeuron.Weights[i] * DifferentYWeight(net, (ActivationNeuron)net.Layers[currentLayer - 1].Neurons[i], arctgFunc, neuronOutputs, differentsY, currentLayer - 1, i, ref activFuncDerivatives, ref isActivFuncDerivativeReady, ref activFuncSecondDerivatives, ref isActivFuncSecondDerivativeReady, layerNumber, neuronInLayerNumber, weightInNeuronNumber, x, y);
                            }
                            return innerWeightDifferent;
                        }
                    }
                    else
                    {
                        double O = 0;
                        if (!isActivFuncDerivativeReady[currentLayer][currentNeuronInCurrentLayer] || !isActivFuncSecondDerivativeReady[currentLayer][currentNeuronInCurrentLayer])
                        {
                            for (int i = 0; i < outputNeuron.InputsCount; i++)
                            {
                                O += outputNeuron.Weights[i] * neuronOutputs[currentLayer - 1][i];
                            }
                            O += outputNeuron.Threshold;
                        }
                        if (!isActivFuncDerivativeReady[currentLayer][currentNeuronInCurrentLayer])
                        {
                            activFuncDerivatives[currentLayer][currentNeuronInCurrentLayer] = arctgFunc.Derivative(O);
                            isActivFuncDerivativeReady[currentLayer][currentNeuronInCurrentLayer] = true;
                        }
                        if (!isActivFuncSecondDerivativeReady[currentLayer][currentNeuronInCurrentLayer])
                        {
                            activFuncSecondDerivatives[currentLayer][currentNeuronInCurrentLayer] = arctgFunc.SecondDerivative(O);
                            isActivFuncSecondDerivativeReady[currentLayer][currentNeuronInCurrentLayer] = true;
                        }
                        double innerDifferent = 0;
                        for (int i = 0; i < outputNeuron.InputsCount; i++)
                        {
                            innerDifferent += outputNeuron.Weights[i] * differentsY[currentLayer - 1][i];
                        }
                        double leftPartInnerWeightDifferent = 0;
                        double rightPartInnerWeightDifferent = 0;
                        if (currentLayer - layerNumber == 1)
                        {
                            leftPartInnerWeightDifferent = outputNeuron.Weights[neuronInLayerNumber] * DifferentWeight(net, (ActivationNeuron)net.Layers[currentLayer - 1].Neurons[neuronInLayerNumber], arctgFunc, neuronOutputs, currentLayer - 1, neuronInLayerNumber, ref activFuncDerivatives, ref isActivFuncDerivativeReady, layerNumber, neuronInLayerNumber, weightInNeuronNumber, x, y);
                            rightPartInnerWeightDifferent = outputNeuron.Weights[neuronInLayerNumber] * DifferentYWeight(net, (ActivationNeuron)net.Layers[currentLayer - 1].Neurons[neuronInLayerNumber], arctgFunc, neuronOutputs, differentsY, currentLayer - 1, neuronInLayerNumber, ref activFuncDerivatives, ref isActivFuncDerivativeReady, ref activFuncSecondDerivatives, ref isActivFuncSecondDerivativeReady, layerNumber, neuronInLayerNumber, weightInNeuronNumber, x, y);
                        }
                        else
                        {
                            for (int i = 0; i < outputNeuron.InputsCount; i++)
                            {
                                leftPartInnerWeightDifferent += outputNeuron.Weights[i] * DifferentWeight(net, (ActivationNeuron)net.Layers[currentLayer - 1].Neurons[i], arctgFunc, neuronOutputs, currentLayer - 1, i, ref activFuncDerivatives, ref isActivFuncDerivativeReady, layerNumber, neuronInLayerNumber, weightInNeuronNumber, x, y);
                                rightPartInnerWeightDifferent += outputNeuron.Weights[i] * DifferentYWeight(net, (ActivationNeuron)net.Layers[currentLayer - 1].Neurons[i], arctgFunc, neuronOutputs, differentsY, currentLayer - 1, i, ref activFuncDerivatives, ref isActivFuncDerivativeReady, ref activFuncSecondDerivatives, ref isActivFuncSecondDerivativeReady, layerNumber, neuronInLayerNumber, weightInNeuronNumber, x, y);
                            }
                        }
                        return activFuncSecondDerivatives[currentLayer][currentNeuronInCurrentLayer] * leftPartInnerWeightDifferent * innerDifferent + activFuncDerivatives[currentLayer][currentNeuronInCurrentLayer] * rightPartInnerWeightDifferent;
                    }
                }
            }
        }

        /// <summary>
        /// Вычисляет производную второй производной f''xx нейросети по заданному весовому коэффициенту в заданной точке.
        /// </summary>
        /// <param name="net">Нейросеть.</param>
        /// <param name="outputNeuron">Заданный нейрон нейросети.</param>
        /// <param name="arctgFunc">Функция активации arctg.</param>
        /// <param name="neuronOutputs">Выходы нейронов нейросети в заданной точке.</param>
        /// <param name="differentsX">Производные f'x выходов нейронов нейросети в заданной точке.</param>
        /// <param name="differentsXX">Вторые производные f''xx выходов нейронов нейросети в заданной точке.</param>
        /// <param name="currentLayer">Номер слоя заданного нейрона.</param>
        /// <param name="currentNeuronInCurrentLayer">Номер заданного нейрона в слое.</param>
        /// <param name="activFuncDerivatives">Производные функции активации arctg от выходов нейронов нейросети в заданной точке.</param>
        /// <param name="isActivFuncDerivativeReady">Показывает, какие производные функции активации arctg от выходов нейронов нейросети в заданной точке уже посчитаны.</param>
        /// <param name="activFuncSecondDerivatives">Вторые производные функции активации arctg от выходов нейронов нейросети в заданной точке.</param>
        /// <param name="isActivFuncSecondDerivativeReady">Показывает, какие вторые производные функции активации arctg от выходов нейронов нейросети в заданной точке уже посчитаны.</param>
        /// <param name="activFuncThirdDerivatives">Третьи производные функции активации arctg от выходов нейронов нейросети в заданной точке.</param>
        /// <param name="isActivFuncThirdDerivativeReady">Показывает, какие третьи производные функции активации arctg от выходов нейронов нейросети в заданной точке уже посчитаны.</param>
        /// <param name="layerNumber">Номер слоя заданного весового коэффициента.</param>
        /// <param name="neuronInLayerNumber">Номер нейрона заданного весового коэффициента в слое.</param>
        /// <param name="weightInNeuronNumber">Номер заданного весового коэффициента в нейроне.</param>
        /// <param name="x">Координата X заданной точки.</param>
        /// <param name="y">Координата Y заданной точки.</param>
        /// <returns>Возвращает вычисленную производную второй производной f''xx нейросети по заданному весовому коэффициенту в заданной точке.</returns>
        static double DifferentXXWeight(ActivationNetwork net, ActivationNeuron outputNeuron, ArctgActivationFunction arctgFunc, double[][] neuronOutputs, double[][] differentsX, double[][] differentsXX, int currentLayer, int currentNeuronInCurrentLayer, ref double[][] activFuncDerivatives, ref bool[][] isActivFuncDerivativeReady, ref double[][] activFuncSecondDerivatives, ref bool[][] isActivFuncSecondDerivativeReady, ref double[][] activFuncThirdDerivatives, ref bool[][] isActivFuncThirdDerivativeReady, int layerNumber, int neuronInLayerNumber, int weightInNeuronNumber, double x, double y)
        {
            if (currentLayer == 0)
            {
                if(!isActivFuncSecondDerivativeReady[currentLayer][currentNeuronInCurrentLayer])
                {
                    activFuncSecondDerivatives[currentLayer][currentNeuronInCurrentLayer] = arctgFunc.SecondDerivative(outputNeuron.Weights[0] * x + outputNeuron.Weights[1] * y + outputNeuron.Threshold);
                    isActivFuncSecondDerivativeReady[currentLayer][currentNeuronInCurrentLayer] = true;
                }
                if (weightInNeuronNumber == 0)
                {
                    return activFuncSecondDerivatives[currentLayer][currentNeuronInCurrentLayer] * outputNeuron.Weights[0] * outputNeuron.Weights[0] * x;
                }
                else if (weightInNeuronNumber == 1)
                {
                    return activFuncSecondDerivatives[currentLayer][currentNeuronInCurrentLayer] * outputNeuron.Weights[0] * outputNeuron.Weights[0] * y;
                }
                else
                {
                    return activFuncSecondDerivatives[currentLayer][currentNeuronInCurrentLayer] * outputNeuron.Weights[0] * outputNeuron.Weights[0];
                }
            }
            else
            {
                if (currentLayer == layerNumber)
                {
                    if (currentLayer == net.Layers.Length - 1)
                    {
                        if (weightInNeuronNumber == outputNeuron.Weights.Length)
                        {
                            return 0;
                        }
                        else
                        {
                            return differentsXX[currentLayer - 1][weightInNeuronNumber];
                        }
                    }
                    else
                    {

                        double O = 0;
                        if(!isActivFuncDerivativeReady[currentLayer][currentNeuronInCurrentLayer] || !isActivFuncSecondDerivativeReady[currentLayer][currentNeuronInCurrentLayer] || !isActivFuncThirdDerivativeReady[currentLayer][currentNeuronInCurrentLayer])
                        {
                            for (int i = 0; i < outputNeuron.InputsCount; i++)
                            {
                                O += outputNeuron.Weights[i] * neuronOutputs[currentLayer - 1][i];
                            }
                            O += outputNeuron.Threshold;
                        }
                        if(!isActivFuncDerivativeReady[currentLayer][currentNeuronInCurrentLayer])
                        {
                            activFuncDerivatives[currentLayer][currentNeuronInCurrentLayer] = arctgFunc.Derivative(O);
                            isActivFuncDerivativeReady[currentLayer][currentNeuronInCurrentLayer] = true;
                        }
                        if(!isActivFuncSecondDerivativeReady[currentLayer][currentNeuronInCurrentLayer])
                        {
                            activFuncSecondDerivatives[currentLayer][currentNeuronInCurrentLayer] = arctgFunc.SecondDerivative(O);
                            isActivFuncSecondDerivativeReady[currentLayer][currentNeuronInCurrentLayer] = true;
                        }
                        if(!isActivFuncThirdDerivativeReady[currentLayer][currentNeuronInCurrentLayer])
                        {
                            activFuncThirdDerivatives[currentLayer][currentNeuronInCurrentLayer] = arctgFunc.ThirdDerivative(O);
                            isActivFuncThirdDerivativeReady[currentLayer][currentNeuronInCurrentLayer] = true;
                        }
                        double innerFirstDifferent = 0;
                        double innerSecondDifferent = 0;
                        for (int i = 0; i < outputNeuron.InputsCount; i++)
                        {
                            innerFirstDifferent += outputNeuron.Weights[i] * differentsX[currentLayer - 1][i];
                            innerSecondDifferent += outputNeuron.Weights[i] * differentsXX[currentLayer - 1][i];
                        }
                        double firstSummand = 0, secondSummand = 0, thirdSummand = 0, fourthSummand = 0;
                        if (weightInNeuronNumber == outputNeuron.Weights.Length)
                        {
                            firstSummand = activFuncThirdDerivatives[currentLayer][currentNeuronInCurrentLayer] * innerFirstDifferent * innerFirstDifferent;
                            thirdSummand = activFuncSecondDerivatives[currentLayer][currentNeuronInCurrentLayer] * innerSecondDifferent;
                        }
                        else
                        {
                            firstSummand = activFuncThirdDerivatives[currentLayer][currentNeuronInCurrentLayer] * neuronOutputs[currentLayer - 1][weightInNeuronNumber] * innerFirstDifferent * innerFirstDifferent;
                            secondSummand = activFuncSecondDerivatives[currentLayer][currentNeuronInCurrentLayer] * 2 * innerFirstDifferent * differentsX[currentLayer - 1][weightInNeuronNumber];
                            thirdSummand = activFuncSecondDerivatives[currentLayer][currentNeuronInCurrentLayer] * neuronOutputs[currentLayer - 1][weightInNeuronNumber] * innerSecondDifferent;
                            fourthSummand = activFuncDerivatives[currentLayer][currentNeuronInCurrentLayer] * differentsXX[currentLayer - 1][weightInNeuronNumber];
                        }
                        return firstSummand + secondSummand + thirdSummand + fourthSummand;
                    }
                }
                else
                {
                    if(currentLayer == net.Layers.Length - 1)
                    {
                        if(currentLayer - layerNumber == 1)
                        {
                            return outputNeuron.Weights[neuronInLayerNumber] * DifferentXXWeight(net, (ActivationNeuron)net.Layers[currentLayer - 1].Neurons[neuronInLayerNumber], arctgFunc, neuronOutputs, differentsX, differentsXX, currentLayer - 1, neuronInLayerNumber, ref activFuncDerivatives, ref isActivFuncDerivativeReady, ref activFuncSecondDerivatives, ref isActivFuncSecondDerivativeReady, ref activFuncThirdDerivatives, ref isActivFuncThirdDerivativeReady, layerNumber, neuronInLayerNumber, weightInNeuronNumber, x, y);
                        }
                        else
                        {
                            double innerSecondWeightDifferent = 0;
                            for(int i = 0; i < outputNeuron.InputsCount; i++)
                            {
                                innerSecondWeightDifferent += outputNeuron.Weights[i] * DifferentXXWeight(net, (ActivationNeuron)net.Layers[currentLayer - 1].Neurons[i], arctgFunc, neuronOutputs, differentsX, differentsXX, currentLayer - 1, i, ref activFuncDerivatives, ref isActivFuncDerivativeReady, ref activFuncSecondDerivatives, ref isActivFuncSecondDerivativeReady, ref activFuncThirdDerivatives, ref isActivFuncThirdDerivativeReady, layerNumber, neuronInLayerNumber, weightInNeuronNumber, x, y);
                            }
                            return innerSecondWeightDifferent;
                        }
                    }
                    else
                    {
                        double O = 0;
                        if (!isActivFuncDerivativeReady[currentLayer][currentNeuronInCurrentLayer] || !isActivFuncSecondDerivativeReady[currentLayer][currentNeuronInCurrentLayer] || !isActivFuncThirdDerivativeReady[currentLayer][currentNeuronInCurrentLayer])
                        {
                            for (int i = 0; i < outputNeuron.InputsCount; i++)
                            {
                                O += outputNeuron.Weights[i] * neuronOutputs[currentLayer - 1][i];
                            }
                            O += outputNeuron.Threshold;
                        }
                        if (!isActivFuncDerivativeReady[currentLayer][currentNeuronInCurrentLayer])
                        {
                            activFuncDerivatives[currentLayer][currentNeuronInCurrentLayer] = arctgFunc.Derivative(O);
                            isActivFuncDerivativeReady[currentLayer][currentNeuronInCurrentLayer] = true;
                        }
                        if (!isActivFuncSecondDerivativeReady[currentLayer][currentNeuronInCurrentLayer])
                        {
                            activFuncSecondDerivatives[currentLayer][currentNeuronInCurrentLayer] = arctgFunc.SecondDerivative(O);
                            isActivFuncSecondDerivativeReady[currentLayer][currentNeuronInCurrentLayer] = true;
                        }
                        if (!isActivFuncThirdDerivativeReady[currentLayer][currentNeuronInCurrentLayer])
                        {
                            activFuncThirdDerivatives[currentLayer][currentNeuronInCurrentLayer] = arctgFunc.ThirdDerivative(O);
                            isActivFuncThirdDerivativeReady[currentLayer][currentNeuronInCurrentLayer] = true;
                        }
                        double innerFirstDifferent = 0;
                        double innerSecondDifferent = 0;
                        for (int i = 0; i < outputNeuron.InputsCount; i++)
                        {
                            innerFirstDifferent += outputNeuron.Weights[i] * differentsX[currentLayer - 1][i];
                            innerSecondDifferent += outputNeuron.Weights[i] * differentsXX[currentLayer - 1][i];
                        }
                        double firstSummand = activFuncThirdDerivatives[currentLayer][currentNeuronInCurrentLayer] * innerFirstDifferent * innerFirstDifferent;
                        double secondSummand = activFuncSecondDerivatives[currentLayer][currentNeuronInCurrentLayer] * 2 * innerFirstDifferent;
                        double thirdSummand = activFuncSecondDerivatives[currentLayer][currentNeuronInCurrentLayer] * innerSecondDifferent;
                        double fourthSummand = activFuncDerivatives[currentLayer][currentNeuronInCurrentLayer];
                        if(currentLayer - layerNumber == 1)
                        {
                            double differentWeight = DifferentWeight(net, (ActivationNeuron)net.Layers[currentLayer - 1].Neurons[neuronInLayerNumber], arctgFunc, neuronOutputs, currentLayer - 1, neuronInLayerNumber, ref activFuncDerivatives, ref isActivFuncDerivativeReady, layerNumber, neuronInLayerNumber, weightInNeuronNumber, x, y);
                            firstSummand *= outputNeuron.Weights[neuronInLayerNumber] * differentWeight;
                            secondSummand *= outputNeuron.Weights[neuronInLayerNumber] * DifferentXWeight(net, (ActivationNeuron)net.Layers[currentLayer - 1].Neurons[neuronInLayerNumber], arctgFunc, neuronOutputs, differentsX, currentLayer - 1, neuronInLayerNumber, ref activFuncDerivatives, ref isActivFuncDerivativeReady, ref activFuncSecondDerivatives, ref isActivFuncSecondDerivativeReady, layerNumber, neuronInLayerNumber, weightInNeuronNumber, x, y);
                            thirdSummand *= outputNeuron.Weights[neuronInLayerNumber] * differentWeight;
                            fourthSummand *= outputNeuron.Weights[neuronInLayerNumber] * DifferentXXWeight(net, (ActivationNeuron)net.Layers[currentLayer - 1].Neurons[neuronInLayerNumber], arctgFunc, neuronOutputs, differentsX, differentsXX, currentLayer - 1, neuronInLayerNumber, ref activFuncDerivatives, ref isActivFuncDerivativeReady, ref activFuncSecondDerivatives, ref isActivFuncSecondDerivativeReady, ref activFuncThirdDerivatives, ref isActivFuncThirdDerivativeReady, layerNumber, neuronInLayerNumber, weightInNeuronNumber, x, y);
                        }
                        else
                        {
                            double additionalDifferentForFirstSummand = 0;
                            double additionalDifferentForSecondSummand = 0;
                            double additionalDifferentForThirdSummand = 0;
                            double additionalDifferentForFourthSummand = 0;
                            for(int i = 0; i < outputNeuron.InputsCount; i++)
                            {
                                double differentWeight = DifferentWeight(net, (ActivationNeuron)net.Layers[currentLayer - 1].Neurons[i], arctgFunc, neuronOutputs, currentLayer - 1, i, ref activFuncDerivatives, ref isActivFuncDerivativeReady, layerNumber, neuronInLayerNumber, weightInNeuronNumber, x, y);
                                additionalDifferentForFirstSummand += outputNeuron.Weights[i] * differentWeight;
                                additionalDifferentForSecondSummand += outputNeuron.Weights[i] * DifferentXWeight(net, (ActivationNeuron)net.Layers[currentLayer - 1].Neurons[i], arctgFunc, neuronOutputs, differentsX, currentLayer - 1, i, ref activFuncDerivatives, ref isActivFuncDerivativeReady, ref activFuncSecondDerivatives, ref isActivFuncSecondDerivativeReady, layerNumber, neuronInLayerNumber, weightInNeuronNumber, x, y);
                                additionalDifferentForThirdSummand += outputNeuron.Weights[i] * differentWeight;
                                additionalDifferentForFourthSummand += outputNeuron.Weights[i] * DifferentXXWeight(net, (ActivationNeuron)net.Layers[currentLayer - 1].Neurons[i], arctgFunc, neuronOutputs, differentsX, differentsXX, currentLayer - 1, i, ref activFuncDerivatives, ref isActivFuncDerivativeReady, ref activFuncSecondDerivatives, ref isActivFuncSecondDerivativeReady, ref activFuncThirdDerivatives, ref isActivFuncThirdDerivativeReady, layerNumber, neuronInLayerNumber, weightInNeuronNumber, x, y);
                            }
                            firstSummand *= additionalDifferentForFirstSummand;
                            secondSummand *= additionalDifferentForSecondSummand;
                            thirdSummand *= additionalDifferentForThirdSummand;
                            fourthSummand *= additionalDifferentForFourthSummand;
                        }
                        return firstSummand + secondSummand + thirdSummand + fourthSummand;
                    }
                }
            }
        }

        /// <summary>
        /// Вычисляет производную второй производной f''xy нейросети по заданному весовому коэффициенту в заданной точке.
        /// </summary>
        /// <param name="net">Нейросеть.</param>
        /// <param name="outputNeuron">Заданный нейрон нейросети.</param>
        /// <param name="arctgFunc">Функция активации arctg.</param>
        /// <param name="neuronOutputs">Выходы нейронов нейросети в заданной точке.</param>
        /// <param name="differentsX">Производные f'x выходов нейронов нейросети в заданной точке.</param>
        /// <param name="differentsXY">Вторые производные f''xy выходов нейронов нейросети в заданной точке.</param>
        /// <param name="currentLayer">Номер слоя заданного нейрона.</param>
        /// <param name="currentNeuronInCurrentLayer">Номер заданного нейрона в слое.</param>
        /// <param name="activFuncDerivatives">Производные функции активации arctg от выходов нейронов нейросети в заданной точке.</param>
        /// <param name="isActivFuncDerivativeReady">Показывает, какие производные функции активации arctg от выходов нейронов нейросети в заданной точке уже посчитаны.</param>
        /// <param name="activFuncSecondDerivatives">Вторые производные функции активации arctg от выходов нейронов нейросети в заданной точке.</param>
        /// <param name="isActivFuncSecondDerivativeReady">Показывает, какие вторые производные функции активации arctg от выходов нейронов нейросети в заданной точке уже посчитаны.</param>
        /// <param name="activFuncThirdDerivatives">Третьи производные функции активации arctg от выходов нейронов нейросети в заданной точке.</param>
        /// <param name="isActivFuncThirdDerivativeReady">Показывает, какие третьи производные функции активации arctg от выходов нейронов нейросети в заданной точке уже посчитаны.</param>
        /// <param name="layerNumber">Номер слоя заданного весового коэффициента.</param>
        /// <param name="neuronInLayerNumber">Номер нейрона заданного весового коэффициента в слое.</param>
        /// <param name="weightInNeuronNumber">Номер заданного весового коэффициента в нейроне.</param>
        /// <param name="x">Координата X заданной точки.</param>
        /// <param name="y">Координата Y заданной точки.</param>
        /// <returns>Возвращает вычисленную производную второй производной f''xy нейросети по заданному весовому коэффициенту в заданной точке.</returns>
        static double DifferentXYWeight(ActivationNetwork net, ActivationNeuron outputNeuron, ArctgActivationFunction arctgFunc, double[][] neuronOutputs, double[][] differentsX, double[][] differentsXY, int currentLayer, int currentNeuronInCurrentLayer, ref double[][] activFuncDerivatives, ref bool[][] isActivFuncDerivativeReady, ref double[][] activFuncSecondDerivatives, ref bool[][] isActivFuncSecondDerivativeReady, ref double[][] activFuncThirdDerivatives, ref bool[][] isActivFuncThirdDerivativeReady, int layerNumber, int neuronInLayerNumber, int weightInNeuronNumber, double x, double y)
        {
            if (currentLayer == 0)
            {
                if (!isActivFuncSecondDerivativeReady[currentLayer][currentNeuronInCurrentLayer])
                {
                    activFuncSecondDerivatives[currentLayer][currentNeuronInCurrentLayer] = arctgFunc.SecondDerivative(outputNeuron.Weights[0] * x + outputNeuron.Weights[1] * y + outputNeuron.Threshold);
                    isActivFuncSecondDerivativeReady[currentLayer][currentNeuronInCurrentLayer] = true;
                }
                if (weightInNeuronNumber == 0)
                {
                    return activFuncSecondDerivatives[currentLayer][currentNeuronInCurrentLayer] * outputNeuron.Weights[0] * outputNeuron.Weights[1] * x;
                }
                else if (weightInNeuronNumber == 1)
                {
                    return activFuncSecondDerivatives[currentLayer][currentNeuronInCurrentLayer] * outputNeuron.Weights[0] * outputNeuron.Weights[1] * y;
                }
                else
                {
                    return activFuncSecondDerivatives[currentLayer][currentNeuronInCurrentLayer] * outputNeuron.Weights[0] * outputNeuron.Weights[1];
                }
            }
            else
            {
                if (currentLayer == layerNumber)
                {
                    if (currentLayer == net.Layers.Length - 1)
                    {
                        if (weightInNeuronNumber == outputNeuron.Weights.Length)
                        {
                            return 0;
                        }
                        else
                        {
                            return differentsXY[currentLayer - 1][weightInNeuronNumber];
                        }
                    }
                    else
                    {

                        double O = 0;
                        if (!isActivFuncDerivativeReady[currentLayer][currentNeuronInCurrentLayer] || !isActivFuncSecondDerivativeReady[currentLayer][currentNeuronInCurrentLayer] || !isActivFuncThirdDerivativeReady[currentLayer][currentNeuronInCurrentLayer])
                        {
                            for (int i = 0; i < outputNeuron.InputsCount; i++)
                            {
                                O += outputNeuron.Weights[i] * neuronOutputs[currentLayer - 1][i];
                            }
                            O += outputNeuron.Threshold;
                        }
                        if (!isActivFuncDerivativeReady[currentLayer][currentNeuronInCurrentLayer])
                        {
                            activFuncDerivatives[currentLayer][currentNeuronInCurrentLayer] = arctgFunc.Derivative(O);
                            isActivFuncDerivativeReady[currentLayer][currentNeuronInCurrentLayer] = true;
                        }
                        if (!isActivFuncSecondDerivativeReady[currentLayer][currentNeuronInCurrentLayer])
                        {
                            activFuncSecondDerivatives[currentLayer][currentNeuronInCurrentLayer] = arctgFunc.SecondDerivative(O);
                            isActivFuncSecondDerivativeReady[currentLayer][currentNeuronInCurrentLayer] = true;
                        }
                        if (!isActivFuncThirdDerivativeReady[currentLayer][currentNeuronInCurrentLayer])
                        {
                            activFuncThirdDerivatives[currentLayer][currentNeuronInCurrentLayer] = arctgFunc.ThirdDerivative(O);
                            isActivFuncThirdDerivativeReady[currentLayer][currentNeuronInCurrentLayer] = true;
                        }
                        double innerFirstDifferent = 0;
                        double innerSecondDifferent = 0;
                        for (int i = 0; i < outputNeuron.InputsCount; i++)
                        {
                            innerFirstDifferent += outputNeuron.Weights[i] * differentsX[currentLayer - 1][i];
                            innerSecondDifferent += outputNeuron.Weights[i] * differentsXY[currentLayer - 1][i];
                        }
                        double firstSummand = 0, secondSummand = 0, thirdSummand = 0, fourthSummand = 0;
                        if (weightInNeuronNumber == outputNeuron.Weights.Length)
                        {
                            firstSummand = activFuncThirdDerivatives[currentLayer][currentNeuronInCurrentLayer] * innerFirstDifferent * innerFirstDifferent;
                            thirdSummand = activFuncSecondDerivatives[currentLayer][currentNeuronInCurrentLayer] * innerSecondDifferent;
                        }
                        else
                        {
                            firstSummand = activFuncThirdDerivatives[currentLayer][currentNeuronInCurrentLayer] * neuronOutputs[currentLayer - 1][weightInNeuronNumber] * innerFirstDifferent * innerFirstDifferent;
                            secondSummand = activFuncSecondDerivatives[currentLayer][currentNeuronInCurrentLayer] * 2 * innerFirstDifferent * differentsX[currentLayer - 1][weightInNeuronNumber];
                            thirdSummand = activFuncSecondDerivatives[currentLayer][currentNeuronInCurrentLayer] * neuronOutputs[currentLayer - 1][weightInNeuronNumber] * innerSecondDifferent;
                            fourthSummand = activFuncDerivatives[currentLayer][currentNeuronInCurrentLayer] * differentsXY[currentLayer - 1][weightInNeuronNumber];
                        }
                        return firstSummand + secondSummand + thirdSummand + fourthSummand;
                    }
                }
                else
                {
                    if (currentLayer == net.Layers.Length - 1)
                    {
                        if (currentLayer - layerNumber == 1)
                        {
                            return outputNeuron.Weights[neuronInLayerNumber] * DifferentXYWeight(net, (ActivationNeuron)net.Layers[currentLayer - 1].Neurons[neuronInLayerNumber], arctgFunc, neuronOutputs, differentsX, differentsXY, currentLayer - 1, neuronInLayerNumber, ref activFuncDerivatives, ref isActivFuncDerivativeReady, ref activFuncSecondDerivatives, ref isActivFuncSecondDerivativeReady, ref activFuncThirdDerivatives, ref isActivFuncThirdDerivativeReady, layerNumber, neuronInLayerNumber, weightInNeuronNumber, x, y);
                        }
                        else
                        {
                            double innerSecondWeightDifferent = 0;
                            for (int i = 0; i < outputNeuron.InputsCount; i++)
                            {
                                innerSecondWeightDifferent += outputNeuron.Weights[i] * DifferentXYWeight(net, (ActivationNeuron)net.Layers[currentLayer - 1].Neurons[i], arctgFunc, neuronOutputs, differentsX, differentsXY, currentLayer - 1, i, ref activFuncDerivatives, ref isActivFuncDerivativeReady, ref activFuncSecondDerivatives, ref isActivFuncSecondDerivativeReady, ref activFuncThirdDerivatives, ref isActivFuncThirdDerivativeReady, layerNumber, neuronInLayerNumber, weightInNeuronNumber, x, y);
                            }
                            return innerSecondWeightDifferent;
                        }
                    }
                    else
                    {
                        double O = 0;
                        if (!isActivFuncDerivativeReady[currentLayer][currentNeuronInCurrentLayer] || !isActivFuncSecondDerivativeReady[currentLayer][currentNeuronInCurrentLayer] || !isActivFuncThirdDerivativeReady[currentLayer][currentNeuronInCurrentLayer])
                        {
                            for (int i = 0; i < outputNeuron.InputsCount; i++)
                            {
                                O += outputNeuron.Weights[i] * neuronOutputs[currentLayer - 1][i];
                            }
                            O += outputNeuron.Threshold;
                        }
                        if (!isActivFuncDerivativeReady[currentLayer][currentNeuronInCurrentLayer])
                        {
                            activFuncDerivatives[currentLayer][currentNeuronInCurrentLayer] = arctgFunc.Derivative(O);
                            isActivFuncDerivativeReady[currentLayer][currentNeuronInCurrentLayer] = true;
                        }
                        if (!isActivFuncSecondDerivativeReady[currentLayer][currentNeuronInCurrentLayer])
                        {
                            activFuncSecondDerivatives[currentLayer][currentNeuronInCurrentLayer] = arctgFunc.SecondDerivative(O);
                            isActivFuncSecondDerivativeReady[currentLayer][currentNeuronInCurrentLayer] = true;
                        }
                        if (!isActivFuncThirdDerivativeReady[currentLayer][currentNeuronInCurrentLayer])
                        {
                            activFuncThirdDerivatives[currentLayer][currentNeuronInCurrentLayer] = arctgFunc.ThirdDerivative(O);
                            isActivFuncThirdDerivativeReady[currentLayer][currentNeuronInCurrentLayer] = true;
                        }
                        double innerFirstDifferent = 0;
                        double innerSecondDifferent = 0;
                        for (int i = 0; i < outputNeuron.InputsCount; i++)
                        {
                            innerFirstDifferent += outputNeuron.Weights[i] * differentsX[currentLayer - 1][i];
                            innerSecondDifferent += outputNeuron.Weights[i] * differentsXY[currentLayer - 1][i];
                        }
                        double firstSummand = activFuncThirdDerivatives[currentLayer][currentNeuronInCurrentLayer] * innerFirstDifferent * innerFirstDifferent;
                        double secondSummand = activFuncSecondDerivatives[currentLayer][currentNeuronInCurrentLayer] * 2 * innerFirstDifferent;
                        double thirdSummand = activFuncSecondDerivatives[currentLayer][currentNeuronInCurrentLayer] * innerSecondDifferent;
                        double fourthSummand = activFuncDerivatives[currentLayer][currentNeuronInCurrentLayer];
                        if (currentLayer - layerNumber == 1)
                        {
                            double differentWeight = DifferentWeight(net, (ActivationNeuron)net.Layers[currentLayer - 1].Neurons[neuronInLayerNumber], arctgFunc, neuronOutputs, currentLayer - 1, neuronInLayerNumber, ref activFuncDerivatives, ref isActivFuncDerivativeReady, layerNumber, neuronInLayerNumber, weightInNeuronNumber, x, y);
                            firstSummand *= outputNeuron.Weights[neuronInLayerNumber] * differentWeight;
                            secondSummand *= outputNeuron.Weights[neuronInLayerNumber] * DifferentXWeight(net, (ActivationNeuron)net.Layers[currentLayer - 1].Neurons[neuronInLayerNumber], arctgFunc, neuronOutputs, differentsX, currentLayer - 1, neuronInLayerNumber, ref activFuncDerivatives, ref isActivFuncDerivativeReady, ref activFuncSecondDerivatives, ref isActivFuncSecondDerivativeReady, layerNumber, neuronInLayerNumber, weightInNeuronNumber, x, y);
                            thirdSummand *= outputNeuron.Weights[neuronInLayerNumber] * differentWeight;
                            fourthSummand *= outputNeuron.Weights[neuronInLayerNumber] * DifferentXYWeight(net, (ActivationNeuron)net.Layers[currentLayer - 1].Neurons[neuronInLayerNumber], arctgFunc, neuronOutputs, differentsX, differentsXY, currentLayer - 1, neuronInLayerNumber, ref activFuncDerivatives, ref isActivFuncDerivativeReady, ref activFuncSecondDerivatives, ref isActivFuncSecondDerivativeReady, ref activFuncThirdDerivatives, ref isActivFuncThirdDerivativeReady, layerNumber, neuronInLayerNumber, weightInNeuronNumber, x, y);
                        }
                        else
                        {
                            double additionalDifferentForFirstSummand = 0;
                            double additionalDifferentForSecondSummand = 0;
                            double additionalDifferentForThirdSummand = 0;
                            double additionalDifferentForFourthSummand = 0;
                            for (int i = 0; i < outputNeuron.InputsCount; i++)
                            {
                                double differentWeight = DifferentWeight(net, (ActivationNeuron)net.Layers[currentLayer - 1].Neurons[i], arctgFunc, neuronOutputs, currentLayer - 1, i, ref activFuncDerivatives, ref isActivFuncDerivativeReady, layerNumber, neuronInLayerNumber, weightInNeuronNumber, x, y);
                                additionalDifferentForFirstSummand += outputNeuron.Weights[i] * differentWeight;
                                additionalDifferentForSecondSummand += outputNeuron.Weights[i] * DifferentXWeight(net, (ActivationNeuron)net.Layers[currentLayer - 1].Neurons[i], arctgFunc, neuronOutputs, differentsX, currentLayer - 1, i, ref activFuncDerivatives, ref isActivFuncDerivativeReady, ref activFuncSecondDerivatives, ref isActivFuncSecondDerivativeReady, layerNumber, neuronInLayerNumber, weightInNeuronNumber, x, y);
                                additionalDifferentForThirdSummand += outputNeuron.Weights[i] * differentWeight;
                                additionalDifferentForFourthSummand += outputNeuron.Weights[i] * DifferentXYWeight(net, (ActivationNeuron)net.Layers[currentLayer - 1].Neurons[i], arctgFunc, neuronOutputs, differentsX, differentsXY, currentLayer - 1, i, ref activFuncDerivatives, ref isActivFuncDerivativeReady, ref activFuncSecondDerivatives, ref isActivFuncSecondDerivativeReady, ref activFuncThirdDerivatives, ref isActivFuncThirdDerivativeReady, layerNumber, neuronInLayerNumber, weightInNeuronNumber, x, y);
                            }
                            firstSummand *= additionalDifferentForFirstSummand;
                            secondSummand *= additionalDifferentForSecondSummand;
                            thirdSummand *= additionalDifferentForThirdSummand;
                            fourthSummand *= additionalDifferentForFourthSummand;
                        }
                        return firstSummand + secondSummand + thirdSummand + fourthSummand;
                    }
                }
            }
        }

        /// <summary>
        /// Вычисляет производную второй производной f''yx нейросети по заданному весовому коэффициенту в заданной точке.
        /// </summary>
        /// <param name="net">Нейросеть.</param>
        /// <param name="outputNeuron">Заданный нейрон нейросети.</param>
        /// <param name="arctgFunc">Функция активации arctg.</param>
        /// <param name="neuronOutputs">Выходы нейронов нейросети в заданной точке.</param>
        /// <param name="differentsY">Производные f'y выходов нейронов нейросети в заданной точке.</param>
        /// <param name="differentsYX">Вторые производные f''yx выходов нейронов нейросети в заданной точке.</param>
        /// <param name="currentLayer">Номер слоя заданного нейрона.</param>
        /// <param name="currentNeuronInCurrentLayer">Номер заданного нейрона в слое.</param>
        /// <param name="activFuncDerivatives">Производные функции активации arctg от выходов нейронов нейросети в заданной точке.</param>
        /// <param name="isActivFuncDerivativeReady">Показывает, какие производные функции активации arctg от выходов нейронов нейросети в заданной точке уже посчитаны.</param>
        /// <param name="activFuncSecondDerivatives">Вторые производные функции активации arctg от выходов нейронов нейросети в заданной точке.</param>
        /// <param name="isActivFuncSecondDerivativeReady">Показывает, какие вторые производные функции активации arctg от выходов нейронов нейросети в заданной точке уже посчитаны.</param>
        /// <param name="activFuncThirdDerivatives">Третьи производные функции активации arctg от выходов нейронов нейросети в заданной точке.</param>
        /// <param name="isActivFuncThirdDerivativeReady">Показывает, какие третьи производные функции активации arctg от выходов нейронов нейросети в заданной точке уже посчитаны.</param>
        /// <param name="layerNumber">Номер слоя заданного весового коэффициента.</param>
        /// <param name="neuronInLayerNumber">Номер нейрона заданного весового коэффициента в слое.</param>
        /// <param name="weightInNeuronNumber">Номер заданного весового коэффициента в нейроне.</param>
        /// <param name="x">Координата X заданной точки.</param>
        /// <param name="y">Координата Y заданной точки.</param>
        /// <returns>Возвращает вычисленную производную второй производной f''yx нейросети по заданному весовому коэффициенту в заданной точке.</returns>
        static double DifferentYXWeight(ActivationNetwork net, ActivationNeuron outputNeuron, ArctgActivationFunction arctgFunc, double[][] neuronOutputs, double[][] differentsY, double[][] differentsYX, int currentLayer, int currentNeuronInCurrentLayer, ref double[][] activFuncDerivatives, ref bool[][] isActivFuncDerivativeReady, ref double[][] activFuncSecondDerivatives, ref bool[][] isActivFuncSecondDerivativeReady, ref double[][] activFuncThirdDerivatives, ref bool[][] isActivFuncThirdDerivativeReady, int layerNumber, int neuronInLayerNumber, int weightInNeuronNumber, double x, double y)
        {
            if (currentLayer == 0)
            {
                if (!isActivFuncSecondDerivativeReady[currentLayer][currentNeuronInCurrentLayer])
                {
                    activFuncSecondDerivatives[currentLayer][currentNeuronInCurrentLayer] = arctgFunc.SecondDerivative(outputNeuron.Weights[0] * x + outputNeuron.Weights[1] * y + outputNeuron.Threshold);
                    isActivFuncSecondDerivativeReady[currentLayer][currentNeuronInCurrentLayer] = true;
                }
                if (weightInNeuronNumber == 0)
                {
                    return activFuncSecondDerivatives[currentLayer][currentNeuronInCurrentLayer] * outputNeuron.Weights[1] * outputNeuron.Weights[0] * x;
                }
                else if (weightInNeuronNumber == 1)
                {
                    return activFuncSecondDerivatives[currentLayer][currentNeuronInCurrentLayer] * outputNeuron.Weights[1] * outputNeuron.Weights[0] * y;
                }
                else
                {
                    return activFuncSecondDerivatives[currentLayer][currentNeuronInCurrentLayer] * outputNeuron.Weights[1] * outputNeuron.Weights[0];
                }
            }
            else
            {
                if (currentLayer == layerNumber)
                {
                    if (currentLayer == net.Layers.Length - 1)
                    {
                        if (weightInNeuronNumber == outputNeuron.Weights.Length)
                        {
                            return 0;
                        }
                        else
                        {
                            return differentsYX[currentLayer - 1][weightInNeuronNumber];
                        }
                    }
                    else
                    {

                        double O = 0;
                        if (!isActivFuncDerivativeReady[currentLayer][currentNeuronInCurrentLayer] || !isActivFuncSecondDerivativeReady[currentLayer][currentNeuronInCurrentLayer] || !isActivFuncThirdDerivativeReady[currentLayer][currentNeuronInCurrentLayer])
                        {
                            for (int i = 0; i < outputNeuron.InputsCount; i++)
                            {
                                O += outputNeuron.Weights[i] * neuronOutputs[currentLayer - 1][i];
                            }
                            O += outputNeuron.Threshold;
                        }
                        if (!isActivFuncDerivativeReady[currentLayer][currentNeuronInCurrentLayer])
                        {
                            activFuncDerivatives[currentLayer][currentNeuronInCurrentLayer] = arctgFunc.Derivative(O);
                            isActivFuncDerivativeReady[currentLayer][currentNeuronInCurrentLayer] = true;
                        }
                        if (!isActivFuncSecondDerivativeReady[currentLayer][currentNeuronInCurrentLayer])
                        {
                            activFuncSecondDerivatives[currentLayer][currentNeuronInCurrentLayer] = arctgFunc.SecondDerivative(O);
                            isActivFuncSecondDerivativeReady[currentLayer][currentNeuronInCurrentLayer] = true;
                        }
                        if (!isActivFuncThirdDerivativeReady[currentLayer][currentNeuronInCurrentLayer])
                        {
                            activFuncThirdDerivatives[currentLayer][currentNeuronInCurrentLayer] = arctgFunc.ThirdDerivative(O);
                            isActivFuncThirdDerivativeReady[currentLayer][currentNeuronInCurrentLayer] = true;
                        }
                        double innerFirstDifferent = 0;
                        double innerSecondDifferent = 0;
                        for (int i = 0; i < outputNeuron.InputsCount; i++)
                        {
                            innerFirstDifferent += outputNeuron.Weights[i] * differentsY[currentLayer - 1][i];
                            innerSecondDifferent += outputNeuron.Weights[i] * differentsYX[currentLayer - 1][i];
                        }
                        double firstSummand = 0, secondSummand = 0, thirdSummand = 0, fourthSummand = 0;
                        if (weightInNeuronNumber == outputNeuron.Weights.Length)
                        {
                            firstSummand = activFuncThirdDerivatives[currentLayer][currentNeuronInCurrentLayer] * innerFirstDifferent * innerFirstDifferent;
                            thirdSummand = activFuncSecondDerivatives[currentLayer][currentNeuronInCurrentLayer] * innerSecondDifferent;
                        }
                        else
                        {
                            firstSummand = activFuncThirdDerivatives[currentLayer][currentNeuronInCurrentLayer] * neuronOutputs[currentLayer - 1][weightInNeuronNumber] * innerFirstDifferent * innerFirstDifferent;
                            secondSummand = activFuncSecondDerivatives[currentLayer][currentNeuronInCurrentLayer] * 2 * innerFirstDifferent * differentsY[currentLayer - 1][weightInNeuronNumber];
                            thirdSummand = activFuncSecondDerivatives[currentLayer][currentNeuronInCurrentLayer] * neuronOutputs[currentLayer - 1][weightInNeuronNumber] * innerSecondDifferent;
                            fourthSummand = activFuncDerivatives[currentLayer][currentNeuronInCurrentLayer] * differentsYX[currentLayer - 1][weightInNeuronNumber];
                        }
                        return firstSummand + secondSummand + thirdSummand + fourthSummand;
                    }
                }
                else
                {
                    if (currentLayer == net.Layers.Length - 1)
                    {
                        if (currentLayer - layerNumber == 1)
                        {
                            return outputNeuron.Weights[neuronInLayerNumber] * DifferentYXWeight(net, (ActivationNeuron)net.Layers[currentLayer - 1].Neurons[neuronInLayerNumber], arctgFunc, neuronOutputs, differentsY, differentsYX, currentLayer - 1, neuronInLayerNumber, ref activFuncDerivatives, ref isActivFuncDerivativeReady, ref activFuncSecondDerivatives, ref isActivFuncSecondDerivativeReady, ref activFuncThirdDerivatives, ref isActivFuncThirdDerivativeReady, layerNumber, neuronInLayerNumber, weightInNeuronNumber, x, y);
                        }
                        else
                        {
                            double innerSecondWeightDifferent = 0;
                            for (int i = 0; i < outputNeuron.InputsCount; i++)
                            {
                                innerSecondWeightDifferent += outputNeuron.Weights[i] * DifferentYXWeight(net, (ActivationNeuron)net.Layers[currentLayer - 1].Neurons[i], arctgFunc, neuronOutputs, differentsY, differentsYX, currentLayer - 1, i, ref activFuncDerivatives, ref isActivFuncDerivativeReady, ref activFuncSecondDerivatives, ref isActivFuncSecondDerivativeReady, ref activFuncThirdDerivatives, ref isActivFuncThirdDerivativeReady, layerNumber, neuronInLayerNumber, weightInNeuronNumber, x, y);
                            }
                            return innerSecondWeightDifferent;
                        }
                    }
                    else
                    {
                        double O = 0;
                        if (!isActivFuncDerivativeReady[currentLayer][currentNeuronInCurrentLayer] || !isActivFuncSecondDerivativeReady[currentLayer][currentNeuronInCurrentLayer] || !isActivFuncThirdDerivativeReady[currentLayer][currentNeuronInCurrentLayer])
                        {
                            for (int i = 0; i < outputNeuron.InputsCount; i++)
                            {
                                O += outputNeuron.Weights[i] * neuronOutputs[currentLayer - 1][i];
                            }
                            O += outputNeuron.Threshold;
                        }
                        if (!isActivFuncDerivativeReady[currentLayer][currentNeuronInCurrentLayer])
                        {
                            activFuncDerivatives[currentLayer][currentNeuronInCurrentLayer] = arctgFunc.Derivative(O);
                            isActivFuncDerivativeReady[currentLayer][currentNeuronInCurrentLayer] = true;
                        }
                        if (!isActivFuncSecondDerivativeReady[currentLayer][currentNeuronInCurrentLayer])
                        {
                            activFuncSecondDerivatives[currentLayer][currentNeuronInCurrentLayer] = arctgFunc.SecondDerivative(O);
                            isActivFuncSecondDerivativeReady[currentLayer][currentNeuronInCurrentLayer] = true;
                        }
                        if (!isActivFuncThirdDerivativeReady[currentLayer][currentNeuronInCurrentLayer])
                        {
                            activFuncThirdDerivatives[currentLayer][currentNeuronInCurrentLayer] = arctgFunc.ThirdDerivative(O);
                            isActivFuncThirdDerivativeReady[currentLayer][currentNeuronInCurrentLayer] = true;
                        }
                        double innerFirstDifferent = 0;
                        double innerSecondDifferent = 0;
                        for (int i = 0; i < outputNeuron.InputsCount; i++)
                        {
                            innerFirstDifferent += outputNeuron.Weights[i] * differentsY[currentLayer - 1][i];
                            innerSecondDifferent += outputNeuron.Weights[i] * differentsYX[currentLayer - 1][i];
                        }
                        double firstSummand = activFuncThirdDerivatives[currentLayer][currentNeuronInCurrentLayer] * innerFirstDifferent * innerFirstDifferent;
                        double secondSummand = activFuncSecondDerivatives[currentLayer][currentNeuronInCurrentLayer] * 2 * innerFirstDifferent;
                        double thirdSummand = activFuncSecondDerivatives[currentLayer][currentNeuronInCurrentLayer] * innerSecondDifferent;
                        double fourthSummand = activFuncDerivatives[currentLayer][currentNeuronInCurrentLayer];
                        if (currentLayer - layerNumber == 1)
                        {
                            double differentWeight = DifferentWeight(net, (ActivationNeuron)net.Layers[currentLayer - 1].Neurons[neuronInLayerNumber], arctgFunc, neuronOutputs, currentLayer - 1, neuronInLayerNumber, ref activFuncDerivatives, ref isActivFuncDerivativeReady, layerNumber, neuronInLayerNumber, weightInNeuronNumber, x, y);
                            firstSummand *= outputNeuron.Weights[neuronInLayerNumber] * differentWeight;
                            secondSummand *= outputNeuron.Weights[neuronInLayerNumber] * DifferentYWeight(net, (ActivationNeuron)net.Layers[currentLayer - 1].Neurons[neuronInLayerNumber], arctgFunc, neuronOutputs, differentsY, currentLayer - 1, neuronInLayerNumber, ref activFuncDerivatives, ref isActivFuncDerivativeReady, ref activFuncSecondDerivatives, ref isActivFuncSecondDerivativeReady, layerNumber, neuronInLayerNumber, weightInNeuronNumber, x, y);
                            thirdSummand *= outputNeuron.Weights[neuronInLayerNumber] * differentWeight;
                            fourthSummand *= outputNeuron.Weights[neuronInLayerNumber] * DifferentYXWeight(net, (ActivationNeuron)net.Layers[currentLayer - 1].Neurons[neuronInLayerNumber], arctgFunc, neuronOutputs, differentsY, differentsYX, currentLayer - 1, neuronInLayerNumber, ref activFuncDerivatives, ref isActivFuncDerivativeReady, ref activFuncSecondDerivatives, ref isActivFuncSecondDerivativeReady, ref activFuncThirdDerivatives, ref isActivFuncThirdDerivativeReady, layerNumber, neuronInLayerNumber, weightInNeuronNumber, x, y);
                        }
                        else
                        {
                            double additionalDifferentForFirstSummand = 0;
                            double additionalDifferentForSecondSummand = 0;
                            double additionalDifferentForThirdSummand = 0;
                            double additionalDifferentForFourthSummand = 0;
                            for (int i = 0; i < outputNeuron.InputsCount; i++)
                            {
                                double differentWeight = DifferentWeight(net, (ActivationNeuron)net.Layers[currentLayer - 1].Neurons[i], arctgFunc, neuronOutputs, currentLayer - 1, i, ref activFuncDerivatives, ref isActivFuncDerivativeReady, layerNumber, neuronInLayerNumber, weightInNeuronNumber, x, y);
                                additionalDifferentForFirstSummand += outputNeuron.Weights[i] * differentWeight;
                                additionalDifferentForSecondSummand += outputNeuron.Weights[i] * DifferentYWeight(net, (ActivationNeuron)net.Layers[currentLayer - 1].Neurons[i], arctgFunc, neuronOutputs, differentsY, currentLayer - 1, i, ref activFuncDerivatives, ref isActivFuncDerivativeReady, ref activFuncSecondDerivatives, ref isActivFuncSecondDerivativeReady, layerNumber, neuronInLayerNumber, weightInNeuronNumber, x, y);
                                additionalDifferentForThirdSummand += outputNeuron.Weights[i] * differentWeight;
                                additionalDifferentForFourthSummand += outputNeuron.Weights[i] * DifferentYXWeight(net, (ActivationNeuron)net.Layers[currentLayer - 1].Neurons[i], arctgFunc, neuronOutputs, differentsY, differentsYX, currentLayer - 1, i, ref activFuncDerivatives, ref isActivFuncDerivativeReady, ref activFuncSecondDerivatives, ref isActivFuncSecondDerivativeReady, ref activFuncThirdDerivatives, ref isActivFuncThirdDerivativeReady, layerNumber, neuronInLayerNumber, weightInNeuronNumber, x, y);
                            }
                            firstSummand *= additionalDifferentForFirstSummand;
                            secondSummand *= additionalDifferentForSecondSummand;
                            thirdSummand *= additionalDifferentForThirdSummand;
                            fourthSummand *= additionalDifferentForFourthSummand;
                        }
                        return firstSummand + secondSummand + thirdSummand + fourthSummand;
                    }
                }
            }
        }

        /// <summary>
        /// Вычисляет производную второй производной f''yy нейросети по заданному весовому коэффициенту в заданной точке.
        /// </summary>
        /// <param name="net">Нейросеть.</param>
        /// <param name="outputNeuron">Заданный нейрон нейросети.</param>
        /// <param name="arctgFunc">Функция активации arctg.</param>
        /// <param name="neuronOutputs">Выходы нейронов нейросети в заданной точке.</param>
        /// <param name="differentsY">Производные f'y выходов нейронов нейросети в заданной точке.</param>
        /// <param name="differentsYY">Вторые производные f''yy выходов нейронов нейросети в заданной точке.</param>
        /// <param name="currentLayer">Номер слоя заданного нейрона.</param>
        /// <param name="currentNeuronInCurrentLayer">Номер заданного нейрона в слое.</param>
        /// <param name="activFuncDerivatives">Производные функции активации arctg от выходов нейронов нейросети в заданной точке.</param>
        /// <param name="isActivFuncDerivativeReady">Показывает, какие производные функции активации arctg от выходов нейронов нейросети в заданной точке уже посчитаны.</param>
        /// <param name="activFuncSecondDerivatives">Вторые производные функции активации arctg от выходов нейронов нейросети в заданной точке.</param>
        /// <param name="isActivFuncSecondDerivativeReady">Показывает, какие вторые производные функции активации arctg от выходов нейронов нейросети в заданной точке уже посчитаны.</param>
        /// <param name="activFuncThirdDerivatives">Третьи производные функции активации arctg от выходов нейронов нейросети в заданной точке.</param>
        /// <param name="isActivFuncThirdDerivativeReady">Показывает, какие третьи производные функции активации arctg от выходов нейронов нейросети в заданной точке уже посчитаны.</param>
        /// <param name="layerNumber">Номер слоя заданного весового коэффициента.</param>
        /// <param name="neuronInLayerNumber">Номер нейрона заданного весового коэффициента в слое.</param>
        /// <param name="weightInNeuronNumber">Номер заданного весового коэффициента в нейроне.</param>
        /// <param name="x">Координата X заданной точки.</param>
        /// <param name="y">Координата Y заданной точки.</param>
        /// <returns>Возвращает вычисленную производную второй производной f''yy нейросети по заданному весовому коэффициенту в заданной точке.</returns>
        static double DifferentYYWeight(ActivationNetwork net, ActivationNeuron outputNeuron, ArctgActivationFunction arctgFunc, double[][] neuronOutputs, double[][] differentsY, double[][] differentsYY, int currentLayer, int currentNeuronInCurrentLayer, ref double[][] activFuncDerivatives, ref bool[][] isActivFuncDerivativeReady, ref double[][] activFuncSecondDerivatives, ref bool[][] isActivFuncSecondDerivativeReady, ref double[][] activFuncThirdDerivatives, ref bool[][] isActivFuncThirdDerivativeReady, int layerNumber, int neuronInLayerNumber, int weightInNeuronNumber, double x, double y)
        {
            if (currentLayer == 0)
            {
                if (!isActivFuncSecondDerivativeReady[currentLayer][currentNeuronInCurrentLayer])
                {
                    activFuncSecondDerivatives[currentLayer][currentNeuronInCurrentLayer] = arctgFunc.SecondDerivative(outputNeuron.Weights[0] * x + outputNeuron.Weights[1] * y + outputNeuron.Threshold);
                    isActivFuncSecondDerivativeReady[currentLayer][currentNeuronInCurrentLayer] = true;
                }
                if (weightInNeuronNumber == 0)
                {
                    return activFuncSecondDerivatives[currentLayer][currentNeuronInCurrentLayer] * outputNeuron.Weights[1] * outputNeuron.Weights[1] * x;
                }
                else if (weightInNeuronNumber == 1)
                {
                    return activFuncSecondDerivatives[currentLayer][currentNeuronInCurrentLayer] * outputNeuron.Weights[1] * outputNeuron.Weights[1] * y;
                }
                else
                {
                    return activFuncSecondDerivatives[currentLayer][currentNeuronInCurrentLayer] * outputNeuron.Weights[1] * outputNeuron.Weights[1];
                }
            }
            else
            {
                if (currentLayer == layerNumber)
                {
                    if (currentLayer == net.Layers.Length - 1)
                    {
                        if (weightInNeuronNumber == outputNeuron.Weights.Length)
                        {
                            return 0;
                        }
                        else
                        {
                            return differentsYY[currentLayer - 1][weightInNeuronNumber];
                        }
                    }
                    else
                    {

                        double O = 0;
                        if (!isActivFuncDerivativeReady[currentLayer][currentNeuronInCurrentLayer] || !isActivFuncSecondDerivativeReady[currentLayer][currentNeuronInCurrentLayer] || !isActivFuncThirdDerivativeReady[currentLayer][currentNeuronInCurrentLayer])
                        {
                            for (int i = 0; i < outputNeuron.InputsCount; i++)
                            {
                                O += outputNeuron.Weights[i] * neuronOutputs[currentLayer - 1][i];
                            }
                            O += outputNeuron.Threshold;
                        }
                        if (!isActivFuncDerivativeReady[currentLayer][currentNeuronInCurrentLayer])
                        {
                            activFuncDerivatives[currentLayer][currentNeuronInCurrentLayer] = arctgFunc.Derivative(O);
                            isActivFuncDerivativeReady[currentLayer][currentNeuronInCurrentLayer] = true;
                        }
                        if (!isActivFuncSecondDerivativeReady[currentLayer][currentNeuronInCurrentLayer])
                        {
                            activFuncSecondDerivatives[currentLayer][currentNeuronInCurrentLayer] = arctgFunc.SecondDerivative(O);
                            isActivFuncSecondDerivativeReady[currentLayer][currentNeuronInCurrentLayer] = true;
                        }
                        if (!isActivFuncThirdDerivativeReady[currentLayer][currentNeuronInCurrentLayer])
                        {
                            activFuncThirdDerivatives[currentLayer][currentNeuronInCurrentLayer] = arctgFunc.ThirdDerivative(O);
                            isActivFuncThirdDerivativeReady[currentLayer][currentNeuronInCurrentLayer] = true;
                        }
                        double innerFirstDifferent = 0;
                        double innerSecondDifferent = 0;
                        for (int i = 0; i < outputNeuron.InputsCount; i++)
                        {
                            innerFirstDifferent += outputNeuron.Weights[i] * differentsY[currentLayer - 1][i];
                            innerSecondDifferent += outputNeuron.Weights[i] * differentsYY[currentLayer - 1][i];
                        }
                        double firstSummand = 0, secondSummand = 0, thirdSummand = 0, fourthSummand = 0;
                        if (weightInNeuronNumber == outputNeuron.Weights.Length)
                        {
                            firstSummand = activFuncThirdDerivatives[currentLayer][currentNeuronInCurrentLayer] * innerFirstDifferent * innerFirstDifferent;
                            thirdSummand = activFuncSecondDerivatives[currentLayer][currentNeuronInCurrentLayer] * innerSecondDifferent;
                        }
                        else
                        {
                            firstSummand = activFuncThirdDerivatives[currentLayer][currentNeuronInCurrentLayer] * neuronOutputs[currentLayer - 1][weightInNeuronNumber] * innerFirstDifferent * innerFirstDifferent;
                            secondSummand = activFuncSecondDerivatives[currentLayer][currentNeuronInCurrentLayer] * 2 * innerFirstDifferent * differentsY[currentLayer - 1][weightInNeuronNumber];
                            thirdSummand = activFuncSecondDerivatives[currentLayer][currentNeuronInCurrentLayer] * neuronOutputs[currentLayer - 1][weightInNeuronNumber] * innerSecondDifferent;
                            fourthSummand = activFuncDerivatives[currentLayer][currentNeuronInCurrentLayer] * differentsYY[currentLayer - 1][weightInNeuronNumber];
                        }
                        return firstSummand + secondSummand + thirdSummand + fourthSummand;
                    }
                }
                else
                {
                    if (currentLayer == net.Layers.Length - 1)
                    {
                        if (currentLayer - layerNumber == 1)
                        {
                            return outputNeuron.Weights[neuronInLayerNumber] * DifferentYYWeight(net, (ActivationNeuron)net.Layers[currentLayer - 1].Neurons[neuronInLayerNumber], arctgFunc, neuronOutputs, differentsY, differentsYY, currentLayer - 1, neuronInLayerNumber, ref activFuncDerivatives, ref isActivFuncDerivativeReady, ref activFuncSecondDerivatives, ref isActivFuncSecondDerivativeReady, ref activFuncThirdDerivatives, ref isActivFuncThirdDerivativeReady, layerNumber, neuronInLayerNumber, weightInNeuronNumber, x, y);
                        }
                        else
                        {
                            double innerSecondWeightDifferent = 0;
                            for (int i = 0; i < outputNeuron.InputsCount; i++)
                            {
                                innerSecondWeightDifferent += outputNeuron.Weights[i] * DifferentYYWeight(net, (ActivationNeuron)net.Layers[currentLayer - 1].Neurons[i], arctgFunc, neuronOutputs, differentsY, differentsYY, currentLayer - 1, i, ref activFuncDerivatives, ref isActivFuncDerivativeReady, ref activFuncSecondDerivatives, ref isActivFuncSecondDerivativeReady, ref activFuncThirdDerivatives, ref isActivFuncThirdDerivativeReady, layerNumber, neuronInLayerNumber, weightInNeuronNumber, x, y);
                            }
                            return innerSecondWeightDifferent;
                        }
                    }
                    else
                    {
                        double O = 0;
                        if (!isActivFuncDerivativeReady[currentLayer][currentNeuronInCurrentLayer] || !isActivFuncSecondDerivativeReady[currentLayer][currentNeuronInCurrentLayer] || !isActivFuncThirdDerivativeReady[currentLayer][currentNeuronInCurrentLayer])
                        {
                            for (int i = 0; i < outputNeuron.InputsCount; i++)
                            {
                                O += outputNeuron.Weights[i] * neuronOutputs[currentLayer - 1][i];
                            }
                            O += outputNeuron.Threshold;
                        }
                        if (!isActivFuncDerivativeReady[currentLayer][currentNeuronInCurrentLayer])
                        {
                            activFuncDerivatives[currentLayer][currentNeuronInCurrentLayer] = arctgFunc.Derivative(O);
                            isActivFuncDerivativeReady[currentLayer][currentNeuronInCurrentLayer] = true;
                        }
                        if (!isActivFuncSecondDerivativeReady[currentLayer][currentNeuronInCurrentLayer])
                        {
                            activFuncSecondDerivatives[currentLayer][currentNeuronInCurrentLayer] = arctgFunc.SecondDerivative(O);
                            isActivFuncSecondDerivativeReady[currentLayer][currentNeuronInCurrentLayer] = true;
                        }
                        if (!isActivFuncThirdDerivativeReady[currentLayer][currentNeuronInCurrentLayer])
                        {
                            activFuncThirdDerivatives[currentLayer][currentNeuronInCurrentLayer] = arctgFunc.ThirdDerivative(O);
                            isActivFuncThirdDerivativeReady[currentLayer][currentNeuronInCurrentLayer] = true;
                        }
                        double innerFirstDifferent = 0;
                        double innerSecondDifferent = 0;
                        for (int i = 0; i < outputNeuron.InputsCount; i++)
                        {
                            innerFirstDifferent += outputNeuron.Weights[i] * differentsY[currentLayer - 1][i];
                            innerSecondDifferent += outputNeuron.Weights[i] * differentsYY[currentLayer - 1][i];
                        }
                        double firstSummand = activFuncThirdDerivatives[currentLayer][currentNeuronInCurrentLayer] * innerFirstDifferent * innerFirstDifferent;
                        double secondSummand = activFuncSecondDerivatives[currentLayer][currentNeuronInCurrentLayer] * 2 * innerFirstDifferent;
                        double thirdSummand = activFuncSecondDerivatives[currentLayer][currentNeuronInCurrentLayer] * innerSecondDifferent;
                        double fourthSummand = activFuncDerivatives[currentLayer][currentNeuronInCurrentLayer];
                        if (currentLayer - layerNumber == 1)
                        {
                            double differentWeight = DifferentWeight(net, (ActivationNeuron)net.Layers[currentLayer - 1].Neurons[neuronInLayerNumber], arctgFunc, neuronOutputs, currentLayer - 1, neuronInLayerNumber, ref activFuncDerivatives, ref isActivFuncDerivativeReady, layerNumber, neuronInLayerNumber, weightInNeuronNumber, x, y);
                            firstSummand *= outputNeuron.Weights[neuronInLayerNumber] * differentWeight;
                            secondSummand *= outputNeuron.Weights[neuronInLayerNumber] * DifferentYWeight(net, (ActivationNeuron)net.Layers[currentLayer - 1].Neurons[neuronInLayerNumber], arctgFunc, neuronOutputs, differentsY, currentLayer - 1, neuronInLayerNumber, ref activFuncDerivatives, ref isActivFuncDerivativeReady, ref activFuncSecondDerivatives, ref isActivFuncSecondDerivativeReady, layerNumber, neuronInLayerNumber, weightInNeuronNumber, x, y);
                            thirdSummand *= outputNeuron.Weights[neuronInLayerNumber] * differentWeight;
                            fourthSummand *= outputNeuron.Weights[neuronInLayerNumber] * DifferentYYWeight(net, (ActivationNeuron)net.Layers[currentLayer - 1].Neurons[neuronInLayerNumber], arctgFunc, neuronOutputs, differentsY, differentsYY, currentLayer - 1, neuronInLayerNumber, ref activFuncDerivatives, ref isActivFuncDerivativeReady, ref activFuncSecondDerivatives, ref isActivFuncSecondDerivativeReady, ref activFuncThirdDerivatives, ref isActivFuncThirdDerivativeReady, layerNumber, neuronInLayerNumber, weightInNeuronNumber, x, y);
                        }
                        else
                        {
                            double additionalDifferentForFirstSummand = 0;
                            double additionalDifferentForSecondSummand = 0;
                            double additionalDifferentForThirdSummand = 0;
                            double additionalDifferentForFourthSummand = 0;
                            for (int i = 0; i < outputNeuron.InputsCount; i++)
                            {
                                double differentWeight = DifferentWeight(net, (ActivationNeuron)net.Layers[currentLayer - 1].Neurons[i], arctgFunc, neuronOutputs, currentLayer - 1, i, ref activFuncDerivatives, ref isActivFuncDerivativeReady, layerNumber, neuronInLayerNumber, weightInNeuronNumber, x, y);
                                additionalDifferentForFirstSummand += outputNeuron.Weights[i] * differentWeight;
                                additionalDifferentForSecondSummand += outputNeuron.Weights[i] * DifferentYWeight(net, (ActivationNeuron)net.Layers[currentLayer - 1].Neurons[i], arctgFunc, neuronOutputs, differentsY, currentLayer - 1, i, ref activFuncDerivatives, ref isActivFuncDerivativeReady, ref activFuncSecondDerivatives, ref isActivFuncSecondDerivativeReady, layerNumber, neuronInLayerNumber, weightInNeuronNumber, x, y);
                                additionalDifferentForThirdSummand += outputNeuron.Weights[i] * differentWeight;
                                additionalDifferentForFourthSummand += outputNeuron.Weights[i] * DifferentYYWeight(net, (ActivationNeuron)net.Layers[currentLayer - 1].Neurons[i], arctgFunc, neuronOutputs, differentsY, differentsYY, currentLayer - 1, i, ref activFuncDerivatives, ref isActivFuncDerivativeReady, ref activFuncSecondDerivatives, ref isActivFuncSecondDerivativeReady, ref activFuncThirdDerivatives, ref isActivFuncThirdDerivativeReady, layerNumber, neuronInLayerNumber, weightInNeuronNumber, x, y);
                            }
                            firstSummand *= additionalDifferentForFirstSummand;
                            secondSummand *= additionalDifferentForSecondSummand;
                            thirdSummand *= additionalDifferentForThirdSummand;
                            fourthSummand *= additionalDifferentForFourthSummand;
                        }
                        return firstSummand + secondSummand + thirdSummand + fourthSummand;
                    }
                }
            }
        }
    }
}
